PAGE ,132                               ;  - Put Printer in 132 Character Mode
TITLE ASMPROCS.ASM - PreProc ASM Routines - Version 1.00
;
;       ASMPROCS is a collection of Assembly Language Routines that perform
;       various Functions for the "CAP" Compiler.
;
;       This Program MUST be Assembled and Linked in .EXE format (it has
;       it's own Code Segment).  This Program should be Linkable with all
;       IBM C/2 Formats, but will NOT be Linkable in a TINY (Same Code and
;       Data Segments) Format - this means this Routine CANNOT be linked
;       into .COM Format.
;
;       Version 1.00                                            Myke Predko
;       Started: 01/16/89
;       Updated: 92.03.19  - Fix GETMEM for > 64K Memory Requests
;       Updated: 92.03.24  - Added "RESIZE" Command
;       Updated: 92.03.27  - Added "ASCIIINT" Command
;       Updated: 92.04.16  - Added "GETKEY" & "GETKEYSTAT" Routines
;       Updated: 94.10.04  - Found Problem with GETMEM > 64K Memory. MAP
;       Updated: 96.02.15  - Got Rid of "STRINGLEN" - Use strlen in "C"
;                97.01.03  - Added "CLI"/"STI" Routines - Turn off/On Ints
;
;       Version 1.00 Contains:
;
;               int GETKEY()           - Return the Current Keystroke
;               int GETKEYSTAT()       - Return the Keyboard Buffer Status
;               int GETROW()           - Return the Cursor Row
;               int GETCOL()           - Return the Cursor Column
;               GOTOXY( int Row, int Col ) - Move the Cursor to Specified Loc'n
;               SCREENCLEAR()          - Clear the Screen
;               long int GETMEMAVAIL() - Return Amount of Memory Available
;               char far *GETMEM( long int MemSize ) - Allocate Specified RAM
;               FREEMEM( void far *FreePtr ) - Free Allocated Storage...
;               RESIZE( void far *ResizePtr, long int MemSize )
;                                      - Resize Allocated Storage...
;               int GETSEG( void far * Ptr ) - Return Pointer Values
;               int GETOFF( void far * Ptr )
;               DECASCII( char far *StringPtr, int Value ) - Int to ASCII
;               int STRSUBCMP( char far *SuperStr, char far *SubString )
;                                      - like C "strcmp" Function
;                                      - Compare to End of SubString
;                                      - Return: < 0 - Super < Sub
;                                      -         = 0 - Super == Sub
;                                      -         > 0 - Super > Sub
;               int STRSUBCMPI( char far *SuperStr, char far *SubString )
;                                      - like C "STRSUBCMP", but Case
;                                         Insensitive
;          long int MINIMUML( long int Value1, long int Value2 )
;                                      - Return the Minimum Value
;                   BREAKPOINT()       - Execute a Break Point Instruction
;                   CLI()              - Turn off Interrupts
;                   STI()              - Turn ON Interrupts
;               int STRINGSEARCH( char far * SuperStr, char far *SubStr )
;                                      - Look for the Specified String
;                   BLOCKMOVE( char far * Dest, char far * Source, int Count )
;                                      - Do a Block Move for Specified Count
;          long int SHORTTOLONG( int i )
;                                      - Convert 16 Bit Number to 32 Bit
;                   HEXBYTE( char far * String, long int Value );
;                                      - Convert Value into Integer
;               int ASCIIINT( char far * String );  - Return the First Two Bytes
;                                         of a String as an Integer
;
PAGE
;
;       CONVENTIONS
;
;       CAPITALS  - CONSTANTS, ASSEMBLER KEYWORDS
;       MixedCase - Variables, Procedure Names, Macros, Structures
;       lowercase - assembler mnemonics
;
;
;       CHAPTERS
;
;       Chapter  1 - Program Information
;       Chapter  2 - Publics and Externals
;       Chapter  3 - Program Segment Information
;       Chapter  4 - Constants
;       Chapter  5 - Structures
;       Chapter  6 - Macros
;       Chapter  7 - "CAP" PreProccessor Subroutines
;       Chapter  8 - Variables
;
PAGE+
;  Define Publics for Subroutines in the Member
        PUBLIC  _GETKEY         ;  Return the Current Keystroke
        PUBLIC  _GETKEYSTAT     ;  Return the Keyboard Buffer Status
        PUBLIC  _GETROW         ;  Return the Cursor Row
        PUBLIC  _GETCOL         ;  Return the Cursor Column
        PUBLIC  _GOTOXY         ;  Move the Cursor to Specified Loc'n
        PUBLIC  _SCREENCLEAR    ;  Clear the Screen
        PUBLIC  _GETMEMAVAIL    ;  Return the Number of Bytes Available for use
        PUBLIC  _GETMEM         ;  Allocate a Specified Amount of Memory
        PUBLIC  _FREEMEM        ;  Release Allocated Memory at Pointer
        PUBLIC  _RESIZE         ;  Resize the Specified Memory
        PUBLIC  _GETSEG         ;  Return the Segment of the Pointer
        PUBLIC  _GETOFF         ;  Return the Offset of the Pointer
        PUBLIC  _DECASCII       ;  Convert the Word in AX to a 5 Digit String
        PUBLIC  _STRSUBCMP      ;  Is the Substring in the String?
        PUBLIC  _STRSUBCMPI     ;  Is the Substring in the String (Case Insens)
        PUBLIC  _MINIMUML       ;  Return the Shortest Long Integer
        PUBLIC  _BREAKPOINT     ;  Execute a Break Instruction (0CCh)
        PUBLIC  _CLI            ;  Turn off Interrupts
        PUBLIC  _STI            ;  Turn ON Interrupts
        PUBLIC  _STRINGSEARCH   ;  Look for the Specified String...
        PUBLIC  _BLOCKMOVE      ;  Move the Block of Data...
        PUBLIC  _SHORTTOLONG    ;  Convert 16 Bit Number to 32
        PUBLIC  _HEXBYTE        ;  Convert Integer into a Hex Byte String
        PUBLIC  _ASCIIINT       ;  Return First Two Bytes of a String as an INT
PAGE+
PASMPROCS SEGMENT               ;  Program Code Segment...
        ASSUME  CS:PASMPROCS
PAGE+
;  Constants
DOSINT  EQU     021h            ;  DOS Access Interrupt
DOSALLOC EQU    048h            ;  Allocate Memory
DOSFREE EQU     049h            ;  Free Allocated Memory
DOSRESIZE EQU   04Ah            ;  Resize Specified Block
MAXPATHLEN EQU  63              ;  Path Length can ONLY be 63 Characters Long

VIDINT  EQU     010h            ;  BIOS Video Interrupt
READCURS EQU    003h            ;  Read the Cursor Position
MOVECURS EQU    002h            ;  Set the Cursor Position
VIDWRITE EQU    009h            ;  Write Attribute/Byte at Cursor

;  Keyboard Constants
KEYINT  EQU     016h            ;  BIOS Keyboard Interrupt
GETKEY  EQU     010h            ;  Return the Entered Key
GETSTAT EQU     011h            ;  Return the Keyboard Status
PAGE+
;  Structures
PtrFmt  STRUC                   ;  Pointer Structure Format
PtrOff  dw      ?               ;  Offset
PtrSeg  dw      ?               ;  Segment
PtrFmt  ENDS
PAGE+
;  Macros
PAGE+
;  _GETKEY - Return the First Character in the Keyboard Buffer
_GETKEY PROC    FAR
        ASSUME  CS:PASMPROCS
        mov     ah, GETKEY      ;  Get the Keyboard Character
        int     KEYINT
        ret                     ;  Return to Caller
_GETKEY ENDP
;  _GETKEYSTATUS - Return the Status of Keyboard Buffer (Nothing, Return 0)
_GETKEYSTAT PROC FAR
        ASSUME  CS:PASMPROCS
        mov     ah, GETSTAT     ;  Get the Keyboard Status
        int     KEYINT
        jnz     _GETKEYSTAT_END ;  Key is Available in the Keystroke Buffer
        xor     ax, ax          ;  Else, Clear AX to Signify, No Key
_GETKEYSTAT_END:
        ret                     ;  Return to Caller
_GETKEYSTAT ENDP
PAGE
;  _GETROW - Return the Current Cursor Row in AX
_GETROW PROC    FAR
        ASSUME  CS:PASMPROCS
        push    bx              ;  Save the Changed Registers
        push    cx
        push    dx
        mov     ah, READCURS    ;  Get the Cursor Position
        xor     bx, bx          ;  Just use Video Page 1
        int     VIDINT
        mov     al, dh          ;  Save the Row in AL
        xor     ah, ah          ;  Clear AH
        pop     dx              ;  Restore the Changed Registers
        pop     cx
        pop     bx
        ret                     ;  Return to Caller
_GETROW ENDP
;  _GETCOL - Return the Current Cursor Column in AX
_GETCOL PROC    FAR
        ASSUME  CS:PASMPROCS
        push    bx              ;  Save the Changed Registers
        push    cx
        push    dx
        mov     ah, READCURS    ;  Get the Cursor Position
        xor     bx, bx          ;  Just use Page 1
        int     VIDINT
        mov     al, dl          ;  Save the Column in AL
        xor     ah, ah          ;  Clear AH
        pop     dx              ;  Restore the Changed Registers
        pop     cx
        pop     bx
        ret                     ;  Return to Caller
_GETCOL ENDP
;  _GOTOXY - Move the Cursor to the Specified Position
_GOTOXY PROC    FAR
        ASSUME  CS:PASMPROCS
        push    bp              ;  Setup the Pointer to the Passed Parameters
        mov     bp, sp
        push    ax              ;  Save the Changed Registers
        push    bx
        push    dx
        mov     ax, [bp+6]      ;  Get the Cursor Row to Move to
        mov     dh, al
        mov     ax, [bp+8]      ;  Get the Cursor Column to Move to
        mov     dl, al
        mov     ah, MOVECURS    ;  Get the Cursor Position
        xor     bx, bx          ;  Just use Page 1
        int     VIDINT
        pop     dx              ;  Restore the Changed Registers
        pop     bx
        pop     ax
        mov     sp, bp          ;  Restore the Pointer to the Passed Parameters
        pop     bp
        ret                     ;  Return to Caller
_GOTOXY ENDP
;  _SCREENCLEAR - Clear the Screen
_SCREENCLEAR PROC FAR
        ASSUME  CS:PASMPROCS
        push    ax              ;  Save the Changed Registers
        push    bx
        push    cx
        push    dx
        push    di
        push    es
        mov     ah, MOVECURS    ;  Move the Cursor to the Top Left
        xor     bx, bx          ;  Do on Page 1
        xor     dx, dx          ;  Top Left is 0,0
        int     VIDINT
        mov     ah, VIDWRITE    ;  Write a Whole Bunch of Characters/Attributes
        mov     al, ' '         ;  Print Blanks
        xor     bh, bh          ;  Just use Page 1
        mov     bl, 007h        ;  Make Screen Grey on Black
        mov     cx, 25*80       ;  Fill the Screen
        int     VIDINT
        pop     es              ;  Restore the Changed Registers
        pop     di
        pop     dx
        pop     cx
        pop     bx
        pop     ax
        ret                     ;  Return to Caller
_SCREENCLEAR ENDP
PAGE
;  _GETMEMAVAIL - Return the Number of Bytes Available....
_GETMEMAVAIL PROC FAR
        ASSUME  CS: PASMPROCS
        push    bx              ; Save the Registers that are Used
        push    cx
        mov     ah, DOSALLOC    ;  Has the Routine been Run Once, Already?
        mov     bx, 0A000h      ;  Request 640K - it won't work
        int     DOSINT          ;  Now, Find Out How Much is there...
        mov     dx, bx          ;  BX has the Max Number avail...
        and     dx, 0F000h      ;  Clear out the Extra Bits...
        mov     cx, 12          ;  Shift to the Right by 12 Bits
        shr     dx, cl
        mov     ax, bx          ;  Now, Get the Low Order Size...
        and     ax, 00FFFh      ;  Multiply by 16 to get the Low Order
        mov     cx, 4
        shl     ax, cl          ;  Now DX-AX has a 32 Bit Number
        pop     cx              ;  Restore Changed Registers
        pop     bx
        ret                     ;  Return to Caller
_GETMEMAVAIL ENDP
PAGE
;  _GETMEM - Allocate the Specified Number of Bytes and Return a Pointer
_GETMEM PROC    FAR
        push    bp              ;  Setup the Pointer to the Parms...
        mov     bp, sp
        push    bx              ;  Save the Registers that are Used
        push    cx
        xor     dx, dx          ;  Assume the That there isn't enough RAM...
        mov     bx, [bp+8]      ;  AX Contains the Low Order?
        and     bx, 0FFF8h      ;  > 512K Bytes Requested?
        jnz     _GETMEMEnd      ;  Yes - No way you can get that...
        mov     dx, [bp+8]      ;  Get What Ever is Greater than 64K...
        mov     ax, [bp+6]      ;  Get the Number passed...
        add     ax, 0000Fh      ;  Put on the Full Paragraph
        adc     dx, 0           ;  Make Sure High Order Byte Get's it as well...
        mov     cx, 16          ;  Now Setup the Number of Paragraphs
        div     cx              ;  Divide by One Digit for # of Paragraphs
        mov     bx, ax          ;  Now, Put # of Paragraphs Required in BX
        xor     dx, dx          ;  Make Sure that DX Doesn't Have Anything
        mov     ah, DOSALLOC
        int     DOSINT
        jc      _GETMEMEnd      ;  Error - More than Available - Return Error
        mov     dx, ax          ;  Else Allocated - Return Pointer in DX:AX
_GETMEMEnd:                     ;  Have the Segment of the Block in AX
        xor     ax, ax
        pop     cx              ;  Restore Changed Registers
        pop     bx
        mov     sp, bp          ;  Restore the Parm Print...
        pop     bp
        ret                     ;  Return to Caller
_GETMEM ENDP
PAGE
;  _FREEMEM - release the Allocated Memory at the Pointer...
_FREEMEM PROC   FAR
        push    bp              ;  Setup the Pointer to the Parms...
        mov     bp, sp
        push    ax
        push    es              ;  Save the Registers that are Used
        mov     es, [bp+8]      ;  Get the Segment to be freed
        mov     ah, DOSFREE     ;  Call DOS to Free
        int     DOSINT
        pop     es              ;  Restore changed Registers...
        pop     ax
        mov     sp, bp          ;  Restore the Stack Pointer
        pop     bp
        ret                     ;  Return to Caller
_FREEMEM ENDP
;  _RESIZE - Resize Allocated Memory Block to Size Specified
_RESIZE PROC    FAR
        push    bp              ;  Setup the Pointer to the Parms...
        mov     bp, sp
        push    bx              ;  Save the Registers that are Used
        push    cx
        push    dx
        push    es
        mov     ax, [bp+10]     ;  AX Contains the Low Order of the New Size
        mov     dx, [bp+12]     ;  DX Contains the High Order of the New Size
        add     ax, 15          ;  Get Size to Nearest Available Paragraph
        adc     dx, 0
        mov     cx, 16          ;  Now Setup the Number of Paragraphs
        div     cx              ;  Divide by One Digit for # of Paragraphs
        mov     bx, ax          ;  Now, Put # of Paragraphs Required in BX
        mov     es, [bp+8]      ;  Get the Pointer to the Block
        xor     dx, dx          ;  Make Sure that DX Doesn't Have Anything
        mov     ah, DOSRESIZE
        int     DOSINT
        pop     es              ;  Restore Changed Registers
        pop     dx
        pop     cx
        pop     bx
        mov     sp, bp          ;  Restore the Parm Print...
        pop     bp
        ret                     ;  Return to Caller
_RESIZE ENDP
PAGE
PAGE
;  _GETSEG - Return the Segment of the Pointer
_GETSEG PROC    FAR
        push    bp              ;  Setup the Pointer to the, uh, Pointer
        mov     bp, sp
        mov     ax, [bp+8]      ;  Save the Segment in AX
        mov     sp, bp          ;  Restore the BP Value
        pop     bp
        ret                     ;  Return to Calling Program
_GETSEG ENDP
;  _GETOFF - Return the Offset of the Pointer
_GETOFF PROC    FAR
        push    bp              ;  Setup the Pointer to the, uh, Pointer
        mov     bp, sp
        mov     ax, [bp+6]      ;  Save the Offset in AX
        mov     sp, bp          ;  Restore the BP Value
        pop     bp
        ret                     ;  Return to Calling Program
_GETOFF ENDP
PAGE
;  Convert the Word Specified and Store at the String...
_DECASCII PROC  FAR
        push    bp              ;  Setup the Parameters...
        mov     bp, sp
        push    ax              ;  Save the Changed Registers...
        push    cx
        push    dx
        push    ds
        mov     ax, [bp+10]     ;  Get the Integer to be Converted
        mov     bx, [bp+6]      ;  Get the Pointer to the String
        mov     ds, [bp+8]
        xor     dx, dx
        mov     cx, 10000
        div     cx              ;  Find out how many TENS of thousands...
        add     al, '0'         ;  Convert High Order Digit to ASCII and Save
        mov     ds:[bx], al
        mov     ax, dx          ;  Now Convert the Remainder...
        xor     dx, dx
        mov     cx, 1000
        div     cx              ;  Find out how many thousands...
        add     al, '0'         ;  Convert High Order Digit to ASCII and Save
        mov     ds:[bx+1], al
        mov     ax, dx          ;  Now Convert the Remainder...
        xor     dx, dx
        mov     cx, 100
        div     cx              ;  Do the Hundreds Column
        add     al, '0'
        mov     ds:[bx+2], al
        mov     ax, dx          ;  Now, Do the Tens...
        mov     cx, 10
        cbw
        div     cl
        add     ax, 03030h      ;  Convert The String to ASCII
        mov     ds:[bx+3], ax   ;  Save the String...
        pop     ds              ;  Restore the Changed Registers
        pop     dx
        pop     cx
        pop     ax
        mov     sp, bp          ;  Restore the Caller's Stack
        pop     bp
        ret                     ;  Return to the Calling Program
_DECASCII ENDP
PAGE
;  Find if the SubString is at the START of the Super String - Case Insensitive
_STRSUBCMPI PROC FAR
        push    bp              ;  Setup the Parms Parameter
        mov     bp, sp
        push    cx              ;  Save Changed Registers
        push    di
        push    si
        push    ds
        push    es
        mov     di, [bp+6]      ;  Get the Pointer to the Super String
        mov     es, [bp+8]
        mov     si, [bp+10]     ;  Get the Pointer to the Sub String
        mov     ds, [bp+12]
        xor     ax, ax          ;  Use Register Compare... - AX Super...
        xor     cx, cx          ;  CX is the SubString
_STRSUBCMPILoop:                ;  Loop Back Here to Check Every Char
        mov     al, es:[di]     ;  Get the Super String Value...
        mov     cl, ds:[si]     ;  Get the Sub String Value
        cmp     cx, 0           ;  Are we at the String End?
        jnz     _STRSUBCMPINoEnd
        xor     ax, ax          ;  Yes, Return Zero
        jmp     SHORT _STRSUBCMPIEnd
_STRSUBCMPINoEnd:               ;  Not at the End, Continue...
        inc     di              ;  Point to the Next Character...
        inc     si
        cmp     al, 'a'         ;  Do we have a Lower Case Character?
        jl      _STRSUBCMPISUBCompare   ;  Nope - Compare
        cmp     al, 'z'         ;  Check the High Boundary
        jg      _STRSUBCMPISUBCompare   ;  Not an ASCII Character, uh, Char
        sub     al, 'a'-'A'     ;  Convert to Upper Case
_STRSUBCMPISUBCompare:          ;  Make sure that the SubString is Upper...
        cmp     cl, 'a'         ;  Are we Lower Case?
        jl      _STRSUBCMPICompare      ;  Nope...
        cmp     cl, 'z'         ;  Else are we in Lower Case?
        jg      _STRSUBCMPICompare      ;  Nope...
        sub     cl, 'a'-'A'     ;  Convert to Upper Case
_STRSUBCMPICompare:             ;  Now, Compare AL to CL
        sub     ax, cx          ;  Compare the Two Bytes
        jz      _STRSUBCMPILoop ;  Bytes are Good - Keep Looping
_STRSUBCMPIEnd:                 ;  Program End - Return AX
        pop     es              ;  Restore Changed Registers
        pop     ds
        pop     si
        pop     di
        pop     cx
        mov     sp, bp          ;  Restore the Parms Stack
        pop     bp
        ret
_STRSUBCMPI ENDP
PAGE
;  Find if the SubString is at the START of the Super String
_STRSUBCMP PROC FAR
        push    bp              ;  Setup the Parms Parameter
        mov     bp, sp
        push    cx              ;  Save Changed Registers
        push    di
        push    si
        push    ds
        push    es
        mov     di, [bp+6]      ;  Get the Pointer to the Super String
        mov     es, [bp+8]
        mov     si, [bp+10]     ;  Get the Pointer to the Sub String
        mov     ds, [bp+12]
        xor     ax, ax          ;  Compare the Two Bytes...
        xor     cx, cx
_STRSUBCMPLoop:                 ;  Loop here to check EVERY Byte
        mov     al, ds:[si]     ;  Get the SubString Value
        mov     cl, es:[di]     ;  Get the SuperString Character
        inc     di              ;  Increment Pointers
        inc     si
        cmp     al, 0           ;  Are we at the End of the Loop?
        jz      _STRSUBCMPEnd   ;  Yes - Return to Caller with 0 in AX
        cmp     cx, ax          ;  Are the Values the same?
        jz      _STRSUBCMPLoop  ;  Yes - Keep Looping
        sub     ax, cx          ;  Nope - Set AX to NOT Zero
_STRSUBCMPEnd:                  ;  Program End - Return AX
        pop     es              ;  Restore Changed Registers
        pop     ds
        pop     si
        pop     di
        pop     cx
        mov     sp, bp          ;  Restore the Parms Stack
        pop     bp
        ret
_STRSUBCMP ENDP
PAGE
;  _MINIMUML - Return the Minimum Long Integer Passed to this Routine
_MINIMUML PROC  FAR
        push    bp              ;  Setup the Pointer to the Parms...
        mov     bp, sp
        push    bx              ;  Save the Registers that are Used
        push    cx
        mov     ax, [bp+6]      ;  Get "Value2"
        mov     bx, [bp+8]      ;  AX Contains the Low Order?
        mov     cx, [bp+10]     ;  Get "Value1"
        mov     dx, [bp+12]     ;  CX Contains the Low Order?
        cmp     dx, bx          ;  Which is Lower?
        jc      _MINIMUML_Value1;  Value1 (in CX/DX) is Lower
        jnz     _MINIMUML_Value2;  Value2 (in AX/BX) is Lower
        cmp     cx, ax          ;  High is the Same - Now, Try Lower...
        jnc     _MINIMUML_Value2;  Value2 (in AX/BX) is Lower
_MINIMUML_Value1:               ;  Return Value in CX/DX
        mov     ax, cx          ;  Get the Lower Word
        jmp     SHORT _MINIMUMEnd       ;  Return to Caller
_MINIMUML_Value2:               ;  Value2 (AX/BX) is Lower...
        mov     dx, bx          ;  Get the Upper Word
_MINIMUMEnd:                    ;  AX/DX has the Smaller Word - Return
        pop     cx              ;  Restore Changed Registers
        pop     bx
        mov     sp, bp          ;  Restore Passed Variable Pointer...
        pop     bp
        ret                     ;  Return to Caller
_MINIMUML ENDP
PAGE
;  _BREAKPOINT - Execute a Break Instruction (Int 03h - 0CCh)
_BREAKPOINT PROC FAR
        int     3               ;  Stop Execution...
        ret                     ;  Return to Caller
_BREAKPOINT ENDP
;  _CLI - Turn off Interrupts
_CLI    PROC FAR
        cli
        ret                     ;  Return to Caller
_CLI    ENDP
;  _STI - Turn ON Interrupts
_STI    PROC FAR
        sti
        ret                     ;  Return to Caller
_STI    ENDP
PAGE
;  Find if the Specified SubString in the SuperString...
_STRINGSEARCH PROC FAR
        push    bp              ;  Setup the Parms Parameter
        mov     bp, sp
        push    bx              ;  Save Changed Registers
        push    cx
        push    dx
        push    di
        push    si
        push    ds
        push    es
        mov     di, [bp+6]      ;  Get the Pointer to the Super String
        mov     es, [bp+8]
        mov     si, [bp+10]     ;  Get the Pointer to the SubString
        mov     ds, [bp+12]
        xor     ax, ax          ;  Use Register Compare... - AX Super...
        xor     cx, cx          ;  CX is the SubString
        xor     bx, bx          ;  BX is the Offset to the Start
        mov     dx, si          ;  Save the Pointer to the SubString
_STRINGSEARCHLoop1:
        push    di              ;  Save the Current Start to the Super Seach
        mov     si, dx          ;  Start at the Start of the SubString
_STRINGSEARCHLoop:              ;  Loop Back Here to Check Every Char
        mov     al, es:[di]     ;  Get the Super String Value...
        mov     cl, ds:[si]     ;  Get the Sub String Value
        inc     di              ;  Point to the Next Character...
        inc     si
        cmp     ax, cx          ;  Compare the Two Bytes
        jnz     _STRINGSEARCHLastChk    ;  Bytes are Good - Keep Looping
        cmp     ax, 0           ;  Are we at the String End?
        jnz     _STRINGSEARCHLoop       ;  Nope - Loop Around and Try Again
_STRINGSEARCHLastChk:           ;  Now, Check to See if we Are at the End
        cmp     cx, 0           ;  Are we at the SubString End?
        jnz     _STRINGSEARCHEndCheck   ;  No - No Match...
        pop     ax              ;  Else - Match = Get Offset in Super of String
        mov     ax, bx          ;  Get the Correct Offset...
        jmp     SHORT _STRINGSEARCHEnd  ;  Return to Caller
_STRINGSEARCHEndCheck:          ;  No Match - Are we are the Super String End?
        cmp     ax, 0           ;  Check for End of Super String
        jnz     _STRINGSEARCHNextLoop   ;  Not at End - Test Again...
        pop     ax              ;  Else at end - Simply Return
        mov     ax, 0FFFFh      ;  Return, -1
        jmp     SHORT _STRINGSEARCHEnd
_STRINGSEARCHNextLoop:          ;  Let's Loop Around
        pop     di              ;  Get the Offset Back
        inc     di              ;  Point to the Next Character
        inc     bx              ;  Increment the Offset to the Super String
        jmp     SHORT _STRINGSEARCHLoop1        ;  Loop to the Next Character
_STRINGSEARCHEnd:               ;  AX Contains the Correct Offset in Super
        pop     es              ;  Restore Changed Registers
        pop     ds
        pop     si
        pop     di
        pop     dx
        pop     cx
        pop     bx
        mov     sp, bp          ;  Restore the Parms Stack
        pop     bp
        ret
_STRINGSEARCH ENDP
PAGE
;  Move a Block of Characters
_BLOCKMOVE PROC FAR
        push    bp              ;  Setup the Parms Parameter
        mov     bp, sp
        push    ax              ;  Save Changed Registers
        push    bx
        push    cx
        push    di
        push    si
        push    ds
        push    es
        mov     di, [bp+6]      ;  Get the Pointer to the Destination Buffer
        mov     es, [bp+8]
        mov     si, [bp+10]     ;  Get the Pointer to the Source Buffer
        mov     ds, [bp+12]
        mov     cx, [bp+14]     ;  Get the Count
        mov     ax, es          ;  Are we in the Same Segment?
        mov     bx, ds
        cmp     ax, bx
        jnz     _BLOCKMOVE_Move ;  Nope - Do Any type of Move...
        cmp     di, si          ;  Is the Destination Higher than the Source?
        jc      _BLOCKMOVE_Move ;  Yes - Simply Read Down the Data
        std                     ;  Reverse the Direction Flag
        add     di, cx          ;  add the Offset to the Buffers...
        dec     di              ;  Normalize
        add     si, cx
        dec     si
_BLOCKMOVE_Move:                ;  Do the Block Move
        rep     movsb
        cld                     ;  Make Sure the Direction is Correct
        pop     es              ;  Restore Changed Registers
        pop     ds
        pop     si
        pop     di
        pop     cx
        pop     bx
        pop     ax
        mov     sp, bp          ;  Restore the Parms Stack
        pop     bp
        ret                     ;  Return to Calling Program
_BLOCKMOVE ENDP
PAGE
;  _SHORTTOLONG  Convert 16 Bit Number to 32
_SHORTTOLONG PROC FAR
        push    bp              ;  Setup the Parms Parameter
        mov     bp, sp
        mov     ax, [bp+6]      ;  Get the Integer
        cwd                     ;  Convert to 32 Bit
        mov     sp, bp          ;  Restore the Parms Stack
        pop     bp
        ret                     ;  Return to Calling Program
_SHORTTOLONG ENDP
PAGE
;  Translate an Integer into a two byte Hex String
_HEXBYTE PROC   FAR
        push    bp              ;  Setup the Parms Parameter
        mov     bp, sp
        push    ax              ;  Save Changed Registers
        push    bx
        push    cx
        push    dx
        push    di
        push    si
        push    es
        mov     di, [bp+6]      ;  Get the Pointer to the Destination String
        mov     es, [bp+8]
        mov     bx, [bp+10]     ;  Get the 32 Bit Number
        mov     dx, [bp+12]
        lea     si, HEXTable    ;  Get the Address of the Hex Table
        cmp     dx, 0           ;  Check to see if the Number is too Big
        jz      _HEXBYTE_BHCheck
        xor     bl, bl          ;  It is - Just Return Zero
        jmp     SHORT _HEXBYTE_BLPrint
_HEXBYTE_BHCheck:               ;  BH should also be equal to Zero
        cmp     bh, 0
        jz      _HEXBYTE_BLPrint
        xor     bl, bl          ;  BH is NOT equal to Zero
_HEXBYTE_BLPrint:               ;  Now, Save the Value that's in AL
        push    bx
        mov     cx, 4
        shr     bx, cl
        and     bx, 0000Fh      ;  Clear the Unused Bits
        mov     al, cs:[si+bx]
        mov     es:[di], al
        pop     bx              ;  Now, do the Low Byte...
        and     bx, 0000Fh
        mov     al, cs:[si+bx]
        mov     es:[di+1], al
        pop     es              ;  Restore Changed Registers
        pop     si
        pop     di
        pop     dx
        pop     cx
        pop     bx
        pop     ax
        mov     sp, bp          ;  Restore the Parms Stack
        pop     bp
        ret                     ;  Return to Calling Program
_HEXBYTE ENDP
PAGE
;  Convert First Two Bytes of a String into an Integer
_ASCIIINT PROC  FAR
        push    bp
        mov     bp, sp
        push    si              ;  Only Require ES:DI for ASCIIINT
        push    ds
        lds     si, [bp+6]
        lodsw                   ;  Read in the Word
        pop     ds              ;  Restore the Changed Registers
        pop     si
        mov     sp, bp
        pop     bp
        ret
_ASCIIINT ENDP
PAGE+
;  Variables
HEXTable db     '01234567890ABCDEF'     ;  Offset to HEX Table
PASMPROCS ENDS
        END
