PAGE ,132
TITLE FindCurs - Show a "Block" cursor if both Shift Keys Pressed
;
;  FindCurs - This is a "TSR" program which will indicate the position
;   of the cursor.  This was originally dated 87.12.29, but the original
;   source has been lost.
;
;  This Program is assembled and linked into a Terminate-and-Stay-Resident
;  (TSR) .COM format executable.
;
;
;  Written By: Myke Predko
;
;
;  Version 1.00
;
;  Started: 87.12.29
;  Updated: 98.12.31 - Recreated for "Inside PC Interfacing"
;
;
;  Conventions:
;  lower case - assembler mnemonics, macro names
;  Mixed Case - Variable Names, Program Labels, And Structures
;  UPPER CASE - CONSTANTS AND ASSEMBLER KEYWORDS
;
;  Chapters: (TSR Format)
;  Chapter  1 - Header
;  Chapter  2 - Start (Jump to Init)
;  Chapter  3 - Structures
;  Chapter  4 - Macros
;  Chapter  5 - Constants
;  Chapter  6 - Variables
;  Chapter  7 - TSR Interrupt Handlers
;  Chapter  8 - TSR Subroutines
;  Chapter 10 - Mainline Code (Setup Program and go TSR)
;  Chapter 11 - Messages

PAGE +
 .model tiny
 .stack
 .code
ProgSeg SEGMENT PARA PUBLIC 'CODE'
        ASSUME  CS:ProgSeg, DS:ProgSeg, ES:ProgSeg, SS:ProgSeg
        ORG     100H
Start:  jmp     SHORT Mainline  ;  Jump to the Program Initialize

PAGE +
subtitle Program Structures
;
PtrFmt  STRUC                   ;  Format of a Pointer to Memory
PtrOff  dw      ?               ;  Offset to the Pointer
PtrSeg  dw      ?               ;  Segment of the Pointer
PtrFmt  ENDS

PAGE +
subtitle Program Macros
;

PAGE +
subtitle Program Constants
;
DOSINT  = 21h                   ;  DOS Interrupt Vector
DOSPrt  = 09h                   ;  Print String Operation
DOSTSR  = 31h                   ;  DOS Terminate and Stay Resident

PAGE +
subtitle Program Variables
Orig0x008 dd 0                  ;  Original Interrupt 0x00C Handler Vector
OrigCurs  dw 0                  ;  Cursor Value before Change
keyVal    db 0                  ;  State of Cursor Update

PAGE +
subtitle Interrupt Handlers
Int0x08:                        ;  Timer Interrupt Handler

  pushf                         ;  "Call" the Original Handler
  call  CS:Orig0x008

  push  AX                      ;  Save Changed Context Registers
  push  CX
  push  DS
  push  ES

  push  CS                      ;  DS = CS for Application Variables
  pop   DS

  mov   AX, 00040h              ;  ES = 0x0040 for System Variables
  mov   ES, AX

  mov   AL, ES:[00017h]         ;  Check the "Shift" Variable
  and   AL, 3
  or    AL, keyVal              ;  With the Current "KeyVal"
  cmp   AL, 3
  jnz   I0_Lift                 ;  Not Down for the First Time

  mov   AX, ES:[00060h]         ;  Get the Current Cursor Settings
  mov   OrigCurs, AX

  mov   AH, 1                   ;  Set the "Block" Cursor
  mov   CX, 0010Bh
  int   010h

  mov   AH, 4                   ;  Mark Cursor Changed
  mov   keyVal, AH

  jmp   SHORT I0_End            ;  Finished with Changing the Cursor

I0_Lift:                        ;  Check for the Cursor Lifted

  and   al, 4                   ;  Is the Block Being Displayed?
  jnz   SHORT I0_End            ;   No - Skip Over

  mov   cx, OrigCurs            ;  Restore the Cursor Value
  mov   AH, 1
  int   010h

  mov   AH, 0                   ;  Mark that Nothing Else is Active
  mov   keyVal, AH

I0_End:

  pop   ES                      ;  Restore Context Registers and Return
  pop   DS
  pop   CX
  pop   AX

  iret


PAGE +
subtitle TSR Subroutines
;

PAGE +
subtitle Mainline - Setup Program and go TSR
Mainline PROC   NEAR

  xor   AX, AX                  ;  Point to the Interrupt Vector Table
  mov   DS, AX

  les   BX, DS:024h             ;  Save Interrupt 0x008 Vector
  mov   CS:Orig0x008.PtrOff, BX
  mov   CS:Orig0x008.PtrSeg, ES

  cli                           ;  Now, Set the New Vector
  lea   AX, Int0x08
  mov   DS:024h, AX
  mov   DS:026h, CS
  sti

;  Everything Done - Now Terminate and Stay Resident
        lea     AX, Mainline    ;  Get Code Size needed in Paragraphs
        add     AX, 15
        mov     CL, 4
        shr     AX, CL
        mov     DX, AX
        mov     AX, ( DOSTSR * 256 )
        int     DOSINT

PAGE +
subtitle Initialization Messages

Mainline ENDP
ProgSeg ENDS
        END     Start           ;  End of Program