PAGE ,132                               ;  - Put Printer in 132 Character Mode
TITLE Boot2.ASM - Display Boot Information
;
;       This Program is a simple boot sector which displays the Address
;       Information of the PC.  This code is converted into a Binary File.
;
;       Myke Predko
;       Started: 98.11.17
;
PAGE
;
;       CONVENTIONS
;
;       CAPITALS  - CONSTANTS, ASSEMBLER KEYWORDS
;       MixedCase - Variables, Procedure Names, Macros, Structures
;       lowercase - assembler mnemonics
;
;
;       CHAPTERS
;
;       Chapter  1 - Program Information
;       Chapter  2 - Publics and Externals
;       Chapter  3 - Program Segment Information
;       Chapter  4 - Constants
;       Chapter  5 - Structures
;       Chapter  6 - Macros
;       Chapter  7 - Mainline
;       Chapter  8 - Subroutines
;       Chapter  9 - Variables
;


PAGE+
;  Define Publics for Subroutines in the Member


PAGE+
ALLPROCS SEGMENT                ;  Segment for Everything
        ASSUME  CS:ALLPROCS, DS:ALLPROCS, ES:ALLPROCS, SS:ALLPROCS


PAGE+
;  Constants
VIDINT  EQU     010h            ;  BIOS Video Interrupt
READCURS EQU    003h            ;  Read the Cursor Position
MOVECURS EQU    002h            ;  Set the Cursor Position
VIDWRITE EQU    009h            ;  Write Attribute/Byte at Cursor


PAGE+
;  Structures
PtrFmt  STRUC                   ;  Pointer Structure Format
PtrOff  dw      ?               ;  Offset
PtrSeg  dw      ?               ;  Segment
PtrFmt  ENDS


PAGE+
;  Macros


PAGE+
;  _GETKEY - Return the First Character in the Keyboard Buffer
 org 0                          ;  Start at Address 0
;org 0100h                      ;  Start at Address 0100h for .COM
START:

  mov   AH, MOVECURS            ;  Move Cursor to Screen Top
  mov   BH, 0
  xor   DX, DX
  int   VIDINT

  call  GetAddr
  add   SI, CSMsg - $           ;  Display the CS:IP Value
  call  MsgDisplay
  mov   AX, CS
  call  SexDisplay              ;  Display the Code Segment
  mov   AL, ':'                 ;  Put in the Inbetween Character
  call  CharDisplay
  call  GetAddr                 ;  Get the Actual Starting Address
IPValue:                        ;  Actual IP Value
  mov   AX, SI
  lea   BX, IPValue
  sub   AX, BX
  call  SexDisplay

  call  NewLine

  call  GetAddr
  add   SI, SPMsg - $           ;  Display the CS:IP Value
  call  MsgDisplay
  push  SS
  pop   AX
  call  SexDisplay              ;  Display the Code Segment
  mov   AL, ':'                 ;  Put in the Inbetween Character
  call  CharDisplay
  mov   AX, SP                  ;  Where does the Program Start?
  call  SexDisplay

Endless:                        ;  End and Just Loop
  jmp   SHORT Endless


PAGE+
;  Subroutines
GetAddr:                        ;  Get the Address of the Call

  pop   SI

  push  SI

  ret

MsgDisplay:                     ;  Display to the Zero

  mov   AL, CS:[SI]
  cmp   AL, 0                   ;  At the End of the String?
  jz    MD_End
  mov   AH, VIDWRITE
  mov   BH, 0
  mov   BL, 7
  mov   CX, 1
  int   VIDINT

  call  CursIncrement

  inc   SI                      ;  Point to the Next Character

  jmp   MsgDisplay

MD_End:                         ;  At the End of the String

  ret

SexDisplay:                     ;  Display the 16 Bit Value

  push  AX

  xchg  AH, AL                  ;  Print the High Byte First

  call  HEXDisplay              ;  Display the Byte in AL

  pop   AX

  call  HEXDisplay              ;  Display the Low Byte

  ret

HexDisplay:                     ;  Display the Byte in AL as a Byte

  push  AX

  mov   cx, 4                   ;  Do the High Nybble First
  shr   AL, CL
  and   AL, 00Fh                ;  Convert to a Hex Byte
  call  HexChar

  pop   AX
  and   AL, 00Fh                ;  Do the Low Nybble Next
  call  HexChar

  ret

HexChar:                        ;  Display the Hex Byte

  add   AL, 0F6h                ;  Do we have A-F?
  jnc   HC_Skip
  add   AL, 7                   ;  Yes, Add 7 To get a Valid Character
HC_Skip:
  sub   AL, 0F6h - '0'

  call  CharDisplay             ;  Display the ASCII Character

  ret

CharDisplay:                    ;  Display the Character in AL

  mov   AH, VIDWRITE
  mov   BH, 0
  mov   BL, 7
  mov   CX, 1
  int   VIDINT

  call  CursIncrement

  ret

CursIncrement:                  ;  Increment the Current Cursor Postion

  mov   AH, READCURS            ;  Increment the Cursor Position
  mov   BH, 0
  int   VIDINT
  mov   AH, MOVECURS
  inc   DL
  int   VIDINT

  ret

NewLine:                        ;  Move the Cursor Down One Line

  mov   AH, READCURS            ;  Increment the Row/Move Column to Zero
  mov   BH, 0
  int   VIDINT
  mov   AH, MOVECURS
  xor   DL, DL
  inc   DH
  int   VIDINT

  ret


PAGE+
;  Variables
CSMsg   db "CS:IP = ", 0
SPMsg   db "SS:SP = ", 0

ALLPROCS ENDS
        END     START
