//  BootMkr - Put the File "boot1.bin" onto the Boot Sector
//
//  This Program Loads the Specified binary file (or "boot1.bin" by
//   default) and then loads the file onto the boot sector of diskette
//   in "A".
//
//  Myke Predko
//  98.11.18
//

// Defines
#include <stdio.h>
#include <dos.h>
#include <asmprocs.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>

#define DiskInt 0x013           //  Diskette Interrupt
#define DiskReset   0           //  Reset the Drive Subsystem
#define DiskStatus  1           //  Get Status of Previous Operation
#define DiskRead    2           //  Read the Sectors in Memory
#define DiskWrite   3           //  Write to the Disk Sector

//  Global Variables
char Buffer[ 512 ];             //  Data Buffer

//  Subroutines


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

struct SREGS sregs;             //  BIOS Variables
union REGS regs;

int  i, j;

int  handle;                    //  File Handling Variables
long handle_size;

char far * SourceFile;          //  File to Load onto the Diskette


  if ( argc == 1 )
    SourceFile = "boot1.bin";
  else                          //  Else, Use Specified File
    SourceFile = argv[ 1 ];


  if (( handle = open( SourceFile, O_RDONLY | O_BINARY )) == -1 )
    printf( "Unable to Find File \"%s\"\n", SourceFile );
  else {                        //  Read/Load the File

    if (( handle_size = lseek( handle, 0L, SEEK_END )) > 512L ) {
      printf( "File Too Lange to Process\N" );
      close( handle );
    } else {                    //  Put the File on the Diskette

      lseek( handle, 0L, SEEK_SET );
      read( handle, Buffer, handle_size & 0x0FFFF );
      close( handle );          //  Read the Binary File
      for ( i = handle_size; i < 512; i++ )
        Buffer[ i ] = 0x090;


      printf( "Offset      0 1 2 3 4 5 6 7  8 9 A B C D E F  0 1 2 3 4 5 6 7  8 9 A B C D E F\n" );
      for ( i = 0; i < 16; i++ ) {  //  Display What was Read
        printf( "     %3X   ", i * 32 );
        for ( j = 0; j < 32; j++ ) {  //  Display Each Line
          printf( "%2X", Buffer[( i * 32 ) + j ] & 0x0FF );
          if (( j & 0x007 ) == 7 )
            printf( " " );
        }  //  endfor
        printf( "\n" );
      }  //  endfor


      regs.h.ah = DiskReset;    //  Reset Disk Drive A
      regs.h.dl = 0;
      int86( DiskInt, &regs, &regs );

      if ( regs.h.ah != 0 )
        printf( "Error Writing to Diskette\n" );
      else {                    //  Write File onto the Boot Sector

        regs.h.ah = DiskWrite;  //  Write the First Sector
        regs.h.dl = 0;          //   of Disk A
        regs.h.dh = 0;          //  Head 0
        regs.h.ch = 0;          //  Track 0
        regs.h.cl = 1;          //  Sector 1
        regs.h.al = 1;          //  Only 1 Sector
        regs.x.bx = GETOFF( &Buffer[ 0 ]);
        sregs.es = GETSEG( &Buffer[ 0 ]);
        int86x( DiskInt, &regs, &regs, &sregs );

        if ( regs.h.ah != 0 )
          printf( "Error Writing to Diskette\n" );

      }  //  endif
    }  //  endif
  }  //  endif

}  //  End BootMkr