//  BootMkr2 - Put "load.bin" onto Boot Sector Along with .com File
//
//  This Program Loads the Specified binary file (or "boot1.bin" by
//   default) and then loads the file onto the boot sector of diskette
//   in "A".
//
//  The Second Parameter is a .com file and is loaded on the Diskette
//   at the next Address.
//
//  Myke Predko
//  98.11.18
//

// Defines
#include <stdio.h>
#include <dos.h>
#include <asmprocs.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>

#define DiskInt 0x013           //  Diskette Interrupt
#define DiskReset   0           //  Reset the Drive Subsystem
#define DiskStatus  1           //  Get Status of Previous Operation
#define DiskRead    2           //  Read the Sectors in Memory
#define DiskWrite   3           //  Write to the Disk Sector

//  Global Variables
char far * Buffer;              //   For "load" and Application


//  Subroutines


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

struct SREGS sregs;             //  BIOS Variables
union REGS regs;

int  i;
long j;

int  handle;                    //  File Handling Variables
long handle_size;


  printf( "BootMkr2\n" );
  printf( "Create a Diskette with the Ability to Load" );
  printf( " and Boot a .COM Application\n\n" );
  printf( "Myke Predko\n" );

  if ( argc != 2 )
    printf( "No Application to Load on the Diskette Specified\n" );
  else {                        //  Put "Load" and Application on Disk

    Buffer = GETMEM( 0x010000L );  //  Get Memory to Load Application

    handle = open( "load.bin", O_RDONLY | O_BINARY );
    handle_size = lseek( handle, 0L, SEEK_END );
    lseek( handle, 0L, SEEK_SET );
    read( handle, Buffer, handle_size & 0x0FFFF );
    close( handle );            //  Read the "load.bin" File

    for ( i = handle_size; i < 0x0100; i++ )
      Buffer[ i ] = 0x090;      //  Fill First 256 Bytes with nops

    for ( i = 0x0100; i < 0x01FF; i++ )
      Buffer[ i ] = 0x000;      //  Fill the PSP with Zeros

    if (( handle = open( argv[ 1 ], O_RDONLY | O_BINARY )) == -1 )
      printf( "Unable to Find File \"%s\"\n", argv[ 1 ]);
    else                        //  Read/Load the File

      if (( handle_size = lseek( handle, 0L, SEEK_END )) > 8704 )
        printf( "File \"%s\" is Too Large to Load\n", argv[ 1 ]);
      else {                    //  Load the File

        lseek( handle, 0L, SEEK_SET );
        read( handle, &Buffer[ 0x0200 ],
                         MINIMUML( 0x07FFFL, handle_size & 0x0FFFF ));
        close( handle );        //  Read the Application File
        for ( j = handle_size + 0x0200; j < 0x010000; j++ )
          Buffer[ j ] = 0x090;  //  Fill the Rest of the Buffer
                                //   with Nops

        regs.h.ah = DiskReset;  //  Reset Disk Drive A
        regs.h.dl = 0;
        int86( DiskInt, &regs, &regs );

        if ( regs.h.ah != 0 )
          printf( "Error Writing to Diskette\n" );
        else {                  //  Write File onto the Boot Sector

          regs.h.ah = DiskWrite;  //  Write the First Sector
          regs.h.dl = 0;        //   of Disk A
          regs.h.dh = 0;        //  Head 0
          regs.h.ch = 0;        //  Track 0
          regs.h.cl = 1;        //  Sector 1
          regs.h.al = 18;       //  Write "Buffer" to Diskette
          regs.x.bx = GETOFF( &Buffer[ 0 ]);
          sregs.es = GETSEG( &Buffer[ 0 ]);
          int86x( DiskInt, &regs, &regs, &sregs );

          if ( regs.h.ah != 0 )
            printf( "Error Writing to Diskette\n" );

        }  //  endif
      }  //  endif
  }  //  endif

}  //  End BootMkr2
