 title  "Dongle - Use a '509 to Create a Parallel Port 'Dongle'"
;
;  This Application uses a 12C509 to provide a "Dongle" Function to PCrpose
;   programs via one of the PC's Parallel Ports.  The "Dongle" code, upon
;   power up loads a 64 Bit value into File Registers and then enables GP0 as
;   Output if GP1 and GP2 are low.
;
;  Data Bits are shifted out each Time GP1 (_SLCT In) Goes High.
;
;  Myke Predko
;  98.12.12
;
;
;  Hardware Notes:
;   Reset is handled internally to the PIC.
;   The Pic is Running with the internal 4 MHz RC Oscillator
;   Vdd is Provided by the Parallel Port's "_Strobe" Pin
;   Gnd is Provided by the Parallel Port's "_AutoFeed" Pin
;   GP0 (Pin 7) Is the Data Output Pin and is Set to Output Upon GP1 = GP2 = Low
;    GP0 is Connected to "D0" of the Parallel Port
;   GP1 is a select line and is driven by the Parallel Port's "_Init" Pin
;   GP2 is the Data Strobe Pin and it is Driven by the Parallel Port's "_SlctIn" Pin
;
  LIST P=12C509, R=DEC          ;  PIC Runs at 4 MHz
  errorlevel 0,-305
  INCLUDE "p12c509.inc"


;  Register Usage
 CBLOCK 0x007
BData: 8                        ;  Eight Data Bits to Shift Out
 ENDC


;  Output Data
BData0 EQU 0x0AA                ;  32 Check Bits to be Output
BData1 EQU 0x055
BData2 EQU 0x000
BData3 EQU 0x0FF

BData4 EQU 0x012                ;  32 Application "Scramble" Bits
BData5 EQU 0x034
BData6 EQU 0x056
BData7 EQU 0x078


 PAGE
 __CONFIG _CP_OFF & _IntRC_OSC & _MCLRE_OFF  & _WDT_OFF
                                ;  Note that the WatchDog Timer is OFF
;  Code for "Dongle"

 org     0

  movwf	 OSCCAL			;  Save the Oscillator Calibration Value

  movlw	 0x0D7			;  Turn Off the Tmer0 Clock source
  option

Reset_App                       ;  Return Here if Any Changes in Data

  movlw  0x03F                  ;  Make Sure that Everything is Disabled
  tris   GPIO

  movlw  BData0                 ;  Load up the 64 Bit Register with the
  movwf  BData + 0              ;   Bit Patterns to Return
  movlw  BData1
  movwf  BData + 1
  movlw  BData2
  movwf  BData + 2
  movlw  BData3
  movwf  BData + 3

  movlw  BData4
  movwf  BData + 4
  movlw  BData5
  movwf  BData + 5
  movlw  BData6
  movwf  BData + 6
  movlw  BData7
  movwf  BData + 7

Wait_Loop                       ;  Wait For the Correct State
                                ;   With GP1 = GP2 = 0
  rrf    GPIO, w
  andlw  3
  btfss  STATUS, Z
   goto  Wait_Loop

Data_Output                     ;  Can Output the Data

  movlw  0x03E                  ;  Enable Bit 0 For Output
  tris   GPIO

Bit_Output                      ;  Shift Out the "Data"

  rrf    BData + 7              ;  Shift Down the Scramble Bits
  rrf    BData + 6
  rrf    BData + 5
  rrf    BData + 4

  rrf    BData + 3              ;  Shift Down the Check Bits
  rrf    BData + 2
  rrf    BData + 1
  rrf    BData

  rlf    BData, w               ;  Output the Low Bit
  movwf  GPIO

Clock_Loop                      ;  Wait for the Strobe to Go High

  rrf    GPIO, w                ;  Check the Bits
  andlw  3
  btfsc  STATUS, Z              ;  If Zero, keeping polling
   goto  Clock_Loop             ;   Else, Reset t

  xorlw  2                      ;  Is Just the Strobe High?
  btfss  STATUS, Z
   goto  Reset_App              ;   - No, Restart the Application

ClockHigh_Loop                  ;  Wait for the Bit to Go Down

  rrf    GPIO, w                ;  Wait for the Line to Go Low
  andlw  3
  xorlw  2
  btfsc  STATUS, Z
   goto  ClockHigh_Loop

  xorlw  2                      ;  Did Just the Strobe Change?
  btfss  STATUS, Z
   goto  Reset_App              ;  No, Reset the Application

  goto   Bit_Output             ;  Output the Next Bit

  end
