PAGE ,132                               ;  - Put Printer in 132 Character Mode
TITLE Dongle1.ASM - Dongle Read Application
;
;       This .COM application Reads the Expected Dongle at the
;       LPT2 Addresses.  The Values Read in Are stored in "Data".
;
;       Myke Predko
;       Started: 98.12.13
;
PAGE
;
;       CONVENTIONS
;
;       CAPITALS  - CONSTANTS, ASSEMBLER KEYWORDS
;       MixedCase - Variables, Procedure Names, Macros, Structures
;       lowercase - assembler mnemonics
;
;
;       CHAPTERS
;
;       Chapter  1 - Program Information
;       Chapter  2 - Publics and Externals
;       Chapter  3 - Program Segment Information
;       Chapter  4 - Constants
;       Chapter  5 - Structures
;       Chapter  6 - Macros
;       Chapter  7 - Mainline
;       Chapter  8 - Subroutines
;       Chapter  9 - Variables
;


PAGE+
;  Define Publics for Subroutines in the Member


PAGE+
ALLPROCS SEGMENT                ;  Segment for Everything
        ASSUME  CS:ALLPROCS, DS:ALLPROCS, ES:ALLPROCS, SS:ALLPROCS


PAGE+
;  Constants
DOSINT  EQU     021h            ;  MS-DOS API Interrupt
DOSSTRG EQU     009h            ;  DOS String Write
DOSEND  EQU     04Ch            ;  End Program

VIDINT  EQU     010h            ;  BIOS Video Interrupt
READCURS EQU    003h            ;  Read the Cursor Position
MOVECURS EQU    002h            ;  Set the Cursor Position
VIDWRITE EQU    009h            ;  Write Attribute/Byte at Cursor

LPT2     EQU    0278h           ;  LPT2 Base Address
PARDATA  EQU    0               ;  Data Port
PARCNTRL EQU    2               ;  Control Port

Timer   EQU     06Ch            ;  Use the Timer Clock

CR      EQU     00Dh            ;  Carriage Return
LF      EQU     00Ah            ;  Line Feed

DONGLERST  EQU  0E4h            ;  Output Value for Reset/Return
DONGLESET  EQU  0E8h            ;  Output Value When Reading Dongle
DONGLESTRB EQU  0E0h            ;  Output Value to Increment


PAGE+
;  Structures
PtrFmt  STRUC                   ;  Pointer Structure Format
PtrOff  dw      ?               ;  Offset
PtrSeg  dw      ?               ;  Segment
PtrFmt  ENDS


PAGE+
;  Macros


PAGE+
;  Dongle1 - Setup the Correct Variables and Reset the Parallel Port
 org 0100h                      ;  Start at Address 0100h for .COM
START:

  mov   AH, DOSSTRG             ;  Print a Message on the Display
  lea   DX, Msg1
  int   DOSINT

  mov   AX, 00040h              ;  The System Variable Segment
  mov   ES, AX

  mov   DX, LPT2 + PARCNTRL     ;  Enable the Dongle
  mov   AL, DONGLESET
  out   DX, AL

  mov   BX, 06Ch                ;  Time Read Value

  mov   AX, WORD PTR ES:[BX]    ;  Read the Current Time
  add   AX, 7                   ;  Add 7 To It to Get the Power Up Delay
                                ;   (330 msec Minimum)
PwrLoop:
  cmp   AX, WORD PTR ES:[BX]    ;  Is the Time Up?
  jnz   PwrLoop                 ;  If Not Zero, then No

  mov   cx, 64                  ;  Read All 64 Bits
ReadLoop:

  call  DataRead                ;  Read in the Data

  mov   AL, DONGLESTRB          ;  Increment the Clock
  out   DX, AL

  call  Dlay                    ;  Wait for One Clock Delay (55 msecs)

  mov   AL, DONGLESET           ;  Set it up to Loop Again
  out   DX, AL

  call  Dlay                    ;  Wait for One Clock Delay (55 msecs)

  loop  ReadLoop                ;  Do for 32 Bits

  mov   AL, DONGLERST           ;  Turn Off the Dongle
  out   DX, AL

  mov   AH, DOSEND              ;  Return to Command.com
  xor   AL, AL
  int   DOSINT


PAGE+
;  Subroutines
DataRead:                       ;  Read in the Data

  sub   DX, PARCNTRL            ;  Going to Read the Data

  in    AL, DX                  ;  Read the Value Pointed to
  rcr   AL, 1                   ;  Save the Bit

  rcr   [DataRd + 6], 1         ;  Shift Up the First Data Word
  rcr   [DataRd + 4], 1         ;  Do the Next Byte
  rcr   [DataRd + 2], 1         ;  And So On...
  rcr   [DataRd], 1

  add   DX, PARCNTRL            ;  Restore DX to the Control Port

  ret


Dlay:                           ;  Wait for One Clock Delay

  mov   AX, WORD PTR ES:[BX]    ;  Read the Current Time
  inc   AX
DlayLoop:                       ;  Loop Here Until Time's Up
  cmp   AX, WORD PTR ES:[BX]
  jnz   DlayLoop

  ret                           ;  Return to Caller


PAGE+
;  Variables
DataRd  dw 0,0,0,0              ;  Eight Bytes for Read in Data

Msg1    db "Dongle Read", CR, LF, '$'

ALLPROCS ENDS
        END     START
