PAGE ,132                               ;  - Put Printer in 132 Character Mode
TITLE Dongle2.ASM - Dongle Read Application
;
;       This .COM application Reads the Expected Dongle at the
;       LPT2 Addresses.  The Values Read in Are stored in "Data".
;
;       After the Dongle Value is Read, the first 32 Bits are checked
;       Against the "CheckBytes" and if the Result is Zero, then the
;       Remaining Program (that has been Inserted into the Application)
;       is decoded and Moved to the Start of the Application.
;
;       Myke Predko
;       Started: 98.12.14
;
PAGE
;
;       CONVENTIONS
;
;       CAPITALS  - CONSTANTS, ASSEMBLER KEYWORDS
;       MixedCase - Variables, Procedure Names, Macros, Structures
;       lowercase - assembler mnemonics
;
;
;       CHAPTERS
;
;       Chapter  1 - Program Information
;       Chapter  2 - Publics and Externals
;       Chapter  3 - Program Segment Information
;       Chapter  4 - Constants
;       Chapter  5 - Structures
;       Chapter  6 - Macros
;       Chapter  7 - Mainline
;       Chapter  8 - Subroutines
;       Chapter  9 - Variables
;       Chapter 10 - Application
;       Chapter 11 - Decode Code
;


PAGE+
;  Define Publics for Subroutines in the Member


PAGE+
ALLPROCS SEGMENT                ;  Segment for Everything
        ASSUME  CS:ALLPROCS, DS:ALLPROCS, ES:ALLPROCS, SS:ALLPROCS


PAGE+
;  Constants
DOSINT  EQU     021h            ;  MS-DOS API Interrupt
DOSDISP EQU     002h            ;  Display the Character in DL
DOSSTRG EQU     009h            ;  DOS String Write
DOSEND  EQU     04Ch            ;  End Program

VIDINT  EQU     010h            ;  BIOS Video Interrupt
READCURS EQU    003h            ;  Read the Cursor Position
MOVECURS EQU    002h            ;  Set the Cursor Position
VIDWRITE EQU    009h            ;  Write Attribute/Byte at Cursor

LPT2     EQU    0278h           ;  LPT2 Base Address
PARDATA  EQU    0               ;  Data Port
PARCNTRL EQU    2               ;  Control Port

Timer   EQU     06Ch            ;  Use the Timer Clock

BS      EQU     008h            ;  BackSpace
CR      EQU     00Dh            ;  Carriage Return
LF      EQU     00Ah            ;  Line Feed

DONGLERST  EQU  0E4h            ;  Output Value for Reset/Return
DONGLESET  EQU  0E8h            ;  Output Value When Reading Dongle
DONGLESTRB EQU  0E0h            ;  Output Value to Increment


PAGE+
;  Structures
PtrFmt  STRUC                   ;  Pointer Structure Format
PtrOff  dw      ?               ;  Offset
PtrSeg  dw      ?               ;  Segment
PtrFmt  ENDS


PAGE+
;  Macros


PAGE+
;  Dongle1 - Setup the Correct Variables and Reset the Parallel Port
 org 0100h                      ;  Start at Address 0100h for .COM
START:

  mov   AH, DOSSTRG             ;  Print a Message on the Display
  lea   DX, Msg1
  int   DOSINT

  mov   AX, 00040h              ;  The System Variable Segment
  mov   ES, AX

  mov   DX, LPT2 + PARCNTRL     ;  Enable the Dongle
  mov   AL, DONGLESET
  out   DX, AL

  mov   BX, 06Ch                ;  Time Read Value

  mov   AX, WORD PTR ES:[BX]    ;  Read the Current Time
  add   AX, 7                   ;  Add 7 To It to Get the Power Up Delay
                                ;   (330 msec Minimum)
PwrLoop:
  cmp   AX, WORD PTR ES:[BX]    ;  Is the Time Up?
  jnz   PwrLoop                 ;  If Not Zero, then No

  mov   cx, 64                  ;  Read All 64 Bits
ReadLoop:

  call  DataRead                ;  Read in the Data

  mov   AL, DONGLESTRB          ;  Increment the Clock
  out   DX, AL

  call  Dlay                    ;  Wait for One Clock Delay (55 msecs)

  mov   AL, DONGLESET           ;  Set it up to Loop Again
  out   DX, AL

  call  Dlay                    ;  Wait for One Clock Delay (55 msecs)

  push  AX
  mov   AL, CL                  ;  Have we Passed 8 Characters?
  and   AL, 7
  jnz   ReadLoopSkip

  push  DX
  mov   AH, DOSDISP             ;  Delete a Character As Dongle Read
  mov   DL, BS
  int   DOSINT
  mov   AH, DOSDISP
  mov   DL, ' '
  int   DOSINT
  mov   AH, DOSDISP
  mov   DL, BS
  int   DOSINT
  pop   DX

ReadLoopSkip:
  pop   AX
  loop  ReadLoop                ;  Do for 32 Bits

  mov   AL, DONGLERST           ;  Turn Off the Dongle
  out   DX, AL

;  Check the First 32 Bits for 0x0FF00:0x0AA55

  mov   AX, [DataRd]
  xor   AX, 055AAh              ;  Does the Low Byte Match?
  jnz   DongleEnd

  mov   AX, [DataRd + 2]
  xor   AX, 0FF00h
  jnz   DongleEnd

;  Set up the Decode/Execute Registers

  push  DS                      ;  ES = DS for Data Movement
  pop   ES

  mov   DI, 0100h               ;  Overwrite Current Code
  mov   SI, AppStart            ;  With the Inserted Code
  mov   cx, AppSize             ;  Set up the Correct Application Size
  inc   CX                      ;  Add One to Make Sure All Words Accessed
  shr   CX, 1                   ;  Divide by 2 for the Number of Words

  mov   BX, [DataRd + 4]        ;  Get the Decode Values
  mov   BP, [DataRd + 6]

  jmp   AppEnd                  ;  Jump to the Execution End

DongleEnd:                      ;  Mismatch, Don't Execute

  mov   AH, DOSSTRG             ;  Print Error Message on the Display
  lea   DX, Msg2
  int   DOSINT

  mov   AH, DOSEND              ;  Return to Command.com
  xor   AL, AL
  int   DOSINT


PAGE+
;  Subroutines
DataRead:                       ;  Read in the Data

  sub   DX, PARCNTRL            ;  Going to Read the Data

  in    AL, DX                  ;  Read the Value Pointed to
  rcr   AL, 1                   ;  Save the Bit

  rcr   [DataRd + 6], 1         ;  Shift Up the First Data Word
  rcr   [DataRd + 4], 1         ;  Do the Next Byte
  rcr   [DataRd + 2], 1         ;  And So On...
  rcr   [DataRd], 1

  add   DX, PARCNTRL            ;  Restore DX to the Control Port

  ret


Dlay:                           ;  Wait for One Clock Delay

  mov   AX, WORD PTR ES:[BX]    ;  Read the Current Time
  inc   AX
DlayLoop:                       ;  Loop Here Until Time's Up
  cmp   AX, WORD PTR ES:[BX]
  jnz   DlayLoop

  ret                           ;  Return to Caller


PAGE+
;  Variables
DataRd  dw 0,0,0,0              ;  Eight Bytes for Read in Data

Msg1    db "DongRead", '$'
Msg2    db "Dongle Not Installed on LPT2", CR, LF, '$'

DongEnd dw 0100h                ;  End When the Conversion has Completed
AppSize dw 0                    ;  Variable to Be Loaded with App Size
AppEnd  dw DecodeExecute        ;  Variable to be Loaded with Decode/Execute


PAGE+
;  #### - Application Code Goes Here
AppStart:

        db 1,2,3,4,5,6,7,8      ;  Code To Indicate Start of App Data

PAGE+
;  Decode/Execute Code
        db 9, 0Ah, 0Bh, 0Ch, 0Dh, 0Eh, 0Fh, 0

DecodeExecute:

  mov   AX, DI                  ;  Do we Have an Odd/Even Word?
  and   AX, 2                   ;  If Not Zero, Have Odd Word
  jnz   DecodeExecuteOdd

DecodeExecuteEven:              ;  Even Word - Use the First 16 Bits

  mov   AX, BX

  jmp   SHORT DecodeExecuteDo

DecodeExecuteOdd:               ;  Odd Word - use the Second 16 Bits

  mov   AX, BP

DecodeExecuteDo:                ;  Now Convert the Value and Save It

  xor   AX, WORD PTR DS:[SI]    ;  Get the Decoded Byte
  add   SI, 2                   ;  Simulate the "LODSW"
; lodsw                         ;  Get the Straight Byte
  stosw                         ;  Store the Word

  loop  DecodeExecute

  xor   AX, AX                  ;  Zero Out the Registers
  xor   BX, BX
  xor   DX, DX                  ;  CX is Zero By "Loop" Instruction
  xor   DI, DI
  xor   SI, SI
  xor   BP, BP

  jmp   DongEnd                 ;  Jumpt to the Variable Address


ALLPROCS ENDS
        END     START
