//  Grabber4 - Graphically Display What was Received from the Camera
//
//  This is updated to Process the changed range.
//
//  This Application runs the ISA Fram Grabber for "Inside PC
//   Interfacing" using the ISA Prototype card with I/O Address
//   0x0300 set.  After the sampling, the data is displayed on
//   the Screen in Graphics mode 13h.  The Display takes data from
//   0x000 to 0x03F, and displays it in Grey Tones, from 0 to 0x0FF.
//
//  This First Program Reads all the Data of the Grabber and
//   Displays the First 25 Lines in Hex (for 32 Characters).
//
//  The I/O Addresses are:
//
//  0x0301 - Read a Byte from the RAM
//  0x0302 - "Arm" the Frame Grabber to Load the Information
//  0x0303 - Reset the Frame Grabber's Counters
//  0x0304 - Decrement the Column Counter
//  0x0305 - Decrement the Row Counter
//
//  Myke Predko
//  98.11.16
//


//  Defines
#include <stdio.h>
#include <dos.h>
#include <asmprocs.h>


#define ByteRead 0x0301         //  Byte to be Read Out
#define Arm      0x0302         //  "Arm" the Read Out
#define Reset    0x0303         //  Reset the Counters
#define DecCol   0x0304         //  Decrement the Row Counter
#define DecRow   0x0305         //  Decrement the Column Counter

#define VidInt 0x010            //  Video Interrupt

struct pixel                    //  Value to Put on the Screen
{
  char character;               //  The Character
  char colour;                  //  Colour Attributes
};

struct palette                  //  Value to Save Pallet Registers
{
  char Red;
  char Green;
  char Blue;
};


//  Global Variables
char VarBuffer[ 64 ][ 64 ];     //  Buffer of Data
int  far * ClkVar;              //  Pointer to Clock Variable

struct pixel Original[ 2000 ];  //  Array Used for the Original Screen
int  OrigPos, OrigCurs;         //  Save the Cursor Position

struct pixel far * VRAM;        //  Pointer to the Video RAM Buffer

struct palette PalData[ 256 ];  //  Data to Save the Palette In

char far * Pixels;              //  Pointer to Pixels on the Display


//  Subroutines


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

unsigned i, j;                  //  Time/Array Counters

union REGS regs;


  (long) ClkVar = 0x00040006C;  //  Point to the Clock Variable


  inp( Reset );                 //  Reset the Counter

  inp( Arm );                   //  Arm Circuit to Wait for Field

  i = *ClkVar + 2;              //  Wait a Minimum of 55 msecs for
  while ( i != *ClkVar );       //   a full field to be sampled.

  inp( Reset );                 //  Reset the Counter Again

  for ( i = 0; i < 64; i++ ) {  //  Read through the Data

    inp( DecRow );              //  Go to the Previous Row

    for ( j = 0; j < 192; j ++ )
      inp( DecCol );            //  Go to the Start of the Line

    for ( j = 0; j < 64; j++ ) {  //  Read the Data

      inp( DecCol );            //  Point to the Column

      VarBuffer[ 63 - i ][ 63 - j ] = inp( ByteRead );

    }  //  endfor

    inp( DecRow );              //  Go to the End of the Row
    inp( DecRow );
    inp( DecRow );

  }  //  endfor


//  Now, Output the Data Output - Use the Routine From "Graphic4"


  regs.h.ah = 3;                //  Save the Current Cursor Position
  regs.h.bh = 0;
  int86( VidInt, &regs, &regs );
  OrigPos = regs.x.dx;
  OrigCurs = regs.x.cx;

  (long) VRAM = 0x0B8000000L;   //  Save the Current Screen
  for ( i = 0; i < 2000; i++ )
    Original[ i ] = VRAM[ i ];

  regs.h.ah = 0;                //  Display the Graphics Screen
  regs.h.al = 0x013;
  int86( VidInt, &regs, &regs );

  for ( i = 0; i < 256; i++ ) { //  Read/Save the Palette Registers
    regs.h.ah = 0x010;          //  Read the Register Values
    regs.h.al = 0x015;
    regs.x.bx = i;
    int86( VidInt, &regs, &regs );
    PalData[ i ].Red = regs.h.dh;
    PalData[ i ].Green = regs.h.ch;
    PalData[ i ].Blue = regs.h.cl;
    regs.h.dh = regs.h.ch = regs.h.cl = i;
    regs.h.ah = 0x010;          //  Put in the New Palette Value
    regs.h.al = 0x010;
    regs.x.bx = i;
    int86( VidInt, &regs, &regs );
  }  /*  endfor  */

//  #### - Put the Data on the Display

  (long) Pixels = 0x0A0000000L; //  Display the Default Pixel Colours
  for ( i = 0; i < 64; i++ )
    for ( j = 0; j < 64; j++ )
      Pixels[( i * 320 ) + j ] = VarBuffer[ i ][ j ];

//  #### - Put the Data on the Display

  while ( GETKEYSTAT() != 0 )   //  Wait for a KeyPress
    GETKEY();

  while ( GETKEYSTAT() == 0 );  //  Wait for a KeyPress
  GETKEY();                     //  Clear it Out of the Buffer

  for ( i = 192; i < 256; i++ ) {  //  Restore the Pixel Colours
    regs.h.ah = 0x010;          //  Read the Register Values
    regs.h.al = 0x010;
    regs.x.bx = i;
    regs.h.dh = PalData[ i ].Red;
    regs.h.ch = PalData[ i ].Green;
    regs.h.cl = PalData[ i ].Blue;
    int86( VidInt, &regs, &regs );
  }  /*  endfor  */

  regs.h.ah = 0;                //  Restore the Display
  regs.h.al = 3;
  int86( VidInt, &regs, &regs );

  regs.h.ah = 1;                //  Restore the Cursor Type
  regs.x.cx = OrigCurs;
  int86( VidInt, &regs, &regs );

  regs.h.ah = 2;                //  Restore the Cursor Position
  regs.h.bh = 0;
  regs.x.dx = OrigPos;
  int86( VidInt, &regs, &regs );

  for ( i = 0; i < 2000; i++ )  //  Restore the Display Values
    VRAM[ i ] = Original[ i ];

}  //  End Grabber
