 title  "ISADCod8 - Use a 16C64A To Decode Mfg_Port Writes."
;
;  This Program is used for the ISA Decode Controller Used in the "Inside PC Interfacing"
;   book's programmable address decode generator.  The actual device is a PIC16C64A.
;
;  ISADCod8 - Add the Switch for Initial Address 0x0080 and 0x0300
;
;  ISADCod7 - Power the SLI-OEM from the PICMicro (Pin PortA.5)
;
;  ISADCod6 - Get Menu 3 (SRAM Write Enable) Working
;
;  ISADCod5 - Work Through Menu System for ISD.
;             Menu 1.  MFG_PORT Output (and Address Select)
;             Menu 2.  SRAM Address
;             Menu 3.  SRAM Write Enable
;
;  ISADCod4 - Display the Character in the Debug Port
;
;  ISADCod3 - Display an "A" on an SLI-OEM Connected to PORTA.0
;
;  ISADCod2 - Read the Port 80 Writes (using the PSP) and Output to PortC
;
;
;  Hardware Notes:
;   The Pic is Running with a 10 MHz Ceramic Oscillator
;   I/O PortA.0 is used to send Serial Data to the SLI-OEM
;   I/O PortA.1 has a button for Option Select
;   I/O PortA.2 has a button for Menu Select
;   I/O PortA.3 drives the "OR" Gate for Disabling SRAM Writes
;   I/O PortA.4 has a Switch for Initial I/O Address Select
;   I/O PortA.5 is used to Power the SLI-OEM
;   I/O PortB is used for the I/O Space Comparison
;   I/O PortC is used for SRAM Address Comparison
;   I/O PortD & E is Used for the PSP Interface
;
;
;  Myke Predko
;  98.11.16
;
  LIST P=16C64A, R=DEC		;  PIC Runs at 10 MHz
  errorlevel 0,-305
  INCLUDE "p16c64A.inc"


;  Register Usage
 CBLOCK 0x020
Count, Counthi			;  Bit Counter
CharOut				;  Serial Character Out
Temp, TempHi
Menu				;  Menu Selection
PortAddr			;  Menu1 - "MFG_Port" Address
SRAMAddr			;  Menu2 - SRAM Address
 ENDC

;  Flags Definition


;  Macros


 PAGE
 __CONFIG _CP_OFF & _PWRTE_ON  & _WDT_OFF & _HS_OSC
                                ;  Note that the WatchDog Timer is OFF
;  Code for ISADCod8

  org    0

  movlw	 0x028			;  PortA.0 Low for Serial/PortA.3 High to Disable SRAM Writes
  movwf	 PORTA			;  Keep Output Low.  PortA.5 Is High to Drive the SLI-OEM

  movlw	 0x010			;  Want 0x080 Output, Shift Down by 3
  btfss	 PORTA, 4		;  If Bit is Set then Use 0x0080
   movlw 0x060			;   Else, Set 0x0300
  movwf	 PORTB			;  Output For the Comparison
  movwf	 PortAddr		;  Save it as the Port Address

  movlw	 0x080			;  Disable the SRAM
  movwf	 PORTC
  movwf	 SRAMAddr

  movlw	 1			;  Select the First Menu (MFG_Port)
  movwf	 Menu

  bsf	 STATUS, RP0		
  bcf	 TRISA ^ 0x080, 0	;  Enable PORTA.0 for Serial Output
  bcf	 TRISA ^ 0x080, 3	;  Enable PORTA.3 for Disabling SRAM Writes
  bcf	 TRISA ^ 0x080, 5	;  Enable PORTA.5 for Powering the SLI-OEM
  clrf	 TRISB ^ 0x080		;  Enable the PortB (MFG_Port Address Comparison) Output Drivers
  clrf	 TRISC ^ 0x080		;  Enable the PortC (SRAM Address Comparison) Drivers
  bsf	 TRISE ^ 0x080, PSPMODE	;  Enable the PSP Mode Drivers
  bcf	 STATUS, RP0

  call	 LongDlay		;  Wait for the SLI-OEM to Get Set up

Menu_Select			;  Select Which Menu to Use

  decf	 Menu, w
  addwf	 PCL

  goto	 Menu1			;  "MFG_Port" Display
  goto	 Menu2			;  RAM Display
  goto	 Menu3			;  RAM Read/Write Select


;  Menu Subroutines
Menu1				;  Display the "MFG_Port" Display

  movlw	 0x0FE			;  Clear the Display
  call	 SendChar
  movlw	 1			
  call	 SendChar

  movlw	 'M'			;  Output "MFG_PORT" to the LCD
  call	 SendChar
  movlw	 'F'
  call	 SendChar
  movlw	 'G'
  call	 SendChar
  movlw	 '_'
  call	 SendChar
  movlw	 'P'
  call	 SendChar
  movlw	 'O'
  call	 SendChar
  movlw	 'R'
  call	 SendChar
  movlw	 'T'
  call	 SendChar

Menu1_Addr			;  Put in the MFG_Port Address

  movlw	 0x0FE			
  call	 SendChar
  movlw	 0x0C0
  call	 SendChar

  movlw	 'A'			;  Put in "Address 0x0"
  call	 SendChar
  movlw	 'd'
  call	 SendChar
  movlw	 'd'
  call	 SendChar
  movlw	 'r'
  call	 SendChar
  movlw	 'e'
  call	 SendChar
  movlw	 's'
  call	 SendChar
  movlw	 's'
  call	 SendChar
  movlw	 ' '
  call	 SendChar
  movlw	 '0'
  call	 SendChar
  movlw	 'x'
  call	 SendChar
  movlw	 '0'
  call	 SendChar

  swapf	 PortAddr, w		;  Display the Port Address
  andlw	 0x00E			;  Clear MSB 3 Bits
  movwf	 Temp
  bcf	 STATUS, C
  rrf	 Temp, w
  addlw	 '0'			;  Convert the Top Bit to Decimal
  call	 SendChar

  movf	 PortAddr, w		;  Display the Lower 5 Five Bits
  andlw	 0x01F			;  Clear the Top 3 Bits
  movwf	 Temp
  bcf	 STATUS, C
  rlf	 Temp
  rlf	 Temp
  rlf	 Temp, w
  call	 SendHex		;  Display as a Hex Number

Menu1_KeyUp			;  Wait for the Keys to be Released

  movf	 PORTA, w
  andlw	 0x006			;  If Button 1 or 2 is Pressed, Keep Looping
  xorlw	 0x006
  btfss	 STATUS, Z
   goto	 Menu1_KeyUp

  movlw	 0x0C2			;  have to wait 20 msecs for Release
  movwf	 Temp
  movlw	 0x012
  movwf	 TempHi

Menu1_KeyUpWait			;  Wait for the Button to be Released

  movf	 PORTA, w		;  Are the Buttons Still Down?
  andlw	 0x006
  xorlw	 0x006
  btfss	 STATUS, Z
   goto	 Menu1_KeyUp		;  Nope, Loop Around

  movlw	 0x0FF			;  At the End of the Delay?  
  addwf	 Temp
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu1_KeyUpWait

Menu1_Loop:			;  Now, Loop Forever reading the port

  bcf	 STATUS, RP0		;  Make Sure in Bank 0

  btfss	 PORTA, 1		;  Check the Option Button
   goto	 Menu1_Option

  btfss	 PORTA, 2		;  Check the Menu Button
   goto	 Menu1_Menu

  bsf	 STATUS, RP0		;  Go into Page 1 for Port Reads

  btfss	 TRISE ^ 0x080, IBF	;  Was something Read?  
   goto	 Menu1_Loop		;  No, Loop Back

  bcf	 STATUS, RP0		;  Read Data From Port D

  movlw	 0x0FE			;  Set the Cursor to the Right Spot
  call	 SendChar
  movlw	 0x089
  call	 SendChar

  movf	 PORTD, w		;  Output to the Serial Port
  call	 SendHex

  bsf	 STATUS, RP0		;  Go back to Page 1

  bcf	 TRISE ^ 0x080, IBOV	;  Make Sure the Overflow is Cleared

  goto	 Menu1_Loop		;  Display the Character Again

Menu1_Menu			;  Menu Button Pressed

  movlw	 0x01C			;  Is the Button Held Down for 20 msecs?  
  movwf	 TempHi
  movlw	 0x0E7

Menu1_MenuLoop			;  Loop Here for 20 msecs

  btfsc	 PORTA, 2		;  Is the Button Still Pressed?
   goto	 Menu1_Loop		;   No - Go Back to the Other Loop

  addlw	 0x0FF
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu1_MenuLoop

  movlw	 2			;  Note that the Second Menu is to Execute
  movwf	 Menu

  goto	 Menu_Select		;  Jump to the Menu Selection

Menu1_Option			;  Option Button Pressed

  movlw	 0x01C			;  Is the Button Held Down for 20 msecs?  
  movwf	 TempHi
  movlw	 0x0E7

Menu1_OptionLoop		;  Loop Here for 20 msecs

  btfsc	 PORTA, 1		;  Is the Button Still Pressed?
   goto	 Menu1_Loop		;   No - Go Back to the Other Loop

  addlw	 0x0FF
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu1_OptionLoop

  incf	 PortAddr		;  Increment the Port Address Monitored

  btfss	 PortAddr, 7		;  Have we gone above Address 0x03F8?
   goto	 Menu1_OptionLow	;  Check for Something Below 0x0100

  clrf	 PortAddr		;  Set PortAddr to 0x080
  bsf	 PortAddr, 4

  goto	 Menu1_NewAddr

Menu1_OptionLow			;  Check to See if Address Between 0x0000 and 0x00F8

  movf	 PortAddr, w
  andlw	 0x060
  btfss	 STATUS, Z		
   goto	 Menu1_NewAddr		;  The Address is Valid

  clrf	 PortAddr		;  Set Address to 0x0100
  bsf	 PortAddr, 5	

Menu1_NewAddr			;  The Address in "PortAddr" is Valid

  movf	 PortAddr, w
  movwf	 PORTB

  goto	 Menu1_Addr		;  Return to the Caller


Menu2				;  Display the SRAM's Address

  movlw	 0x0FE			;  Clear the Display
  call	 SendChar
  movlw	 1			
  call	 SendChar

  movlw	 'S'			;  Output "SRAM Address" to the LCD
  call	 SendChar
  movlw	 'R'
  call	 SendChar
  movlw	 'A'
  call	 SendChar
  movlw	 'M'
  call	 SendChar
  movlw	 ' '
  call	 SendChar
  movlw	 'A'
  call	 SendChar
  movlw	 'd'
  call	 SendChar
  movlw	 'd'
  call	 SendChar
  movlw	 'r'
  call	 SendChar
  movlw	 'e'
  call	 SendChar
  movlw	 's'
  call	 SendChar
  movlw	 's'
  call	 SendChar

Menu2_Addr			;  Put in the SRAM's Address

  movlw	 0x0FE			
  call	 SendChar
  movlw	 0x0C0
  call	 SendChar

  btfss	 SRAMAddr, 7		;  Is the SRAM Disabled?
   goto	 Menu2_Actual

  movlw	 'D'			;  Yes, Note it
  call	 SendChar
  movlw	 'i'
  call	 SendChar
  movlw	 's'
  call	 SendChar
  movlw	 'a'
  call	 SendChar
  movlw	 'b'
  call	 SendChar
  movlw	 'l'
  call	 SendChar
  movlw	 'e'
  call	 SendChar
  movlw	 'd'
  call	 SendChar

  goto	 Menu2_KeyUp

Menu2_Actual			;  There is an SRAM Address Set

  movlw	 '0'
  call	 SendChar
  movlw	 'x'
  call	 SendChar
  movlw	 '0'
  call	 SendChar

  movf	 SRAMAddr, w		;  Display the Upper 8 Bits
  movwf	 Temp
  bcf	 STATUS, C
  rlf	 Temp, w
  call	 SendHex		;  Display as a Hex Number

  movlw	 '0'			;  Add the last 3 "Zeros"
  call	 SendChar
  movlw	 '0'			
  call	 SendChar
  movlw	 '0'			
  call	 SendChar

Menu2_KeyUp			;  Wait for the Keys to be Released

  movf	 PORTA, w
  andlw	 0x006			;  If Button 1 or 2 is Pressed, Keep Looping
  xorlw	 0x006
  btfss	 STATUS, Z
   goto	 Menu2_KeyUp

  movlw	 0x0C2			;  have to wait 20 msecs for Release
  movwf	 Temp
  movlw	 0x012
  movwf	 TempHi

Menu2_KeyUpWait			;  Wait for the Button to be Released

  movf	 PORTA, w		;  Are the Buttons Still Down?
  andlw	 0x006
  xorlw	 0x006
  btfss	 STATUS, Z
   goto	 Menu2_KeyUp		;  Nope, Loop Around

  movlw	 0x0FF			;  At the End of the Delay?  
  addwf	 Temp
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu2_KeyUpWait

Menu2_Loop:			;  Now, Loop Forever reading the port

  btfss	 PORTA, 1		;  Check the Option Button
   goto	 Menu2_Option

  btfss	 PORTA, 2		;  Check the Menu Button
   goto	 Menu2_Menu

  goto	 Menu2_Loop		;  Loop Around Again, Polling Buttons

Menu2_Menu			;  Menu Button Pressed

  movlw	 0x01C			;  Is the Button Held Down for 20 msecs?  
  movwf	 TempHi
  movlw	 0x0E7

Menu2_MenuLoop			;  Loop Here for 20 msecs

  btfsc	 PORTA, 2		;  Is the Button Still Pressed?
   goto	 Menu2_Loop		;   No - Go Back to the Other Loop

  addlw	 0x0FF
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu2_MenuLoop

  movlw	 3			;  Note that the Second Menu is to Execute
  movwf	 Menu

  goto	 Menu_Select		;  Jump to the Menu Selection

Menu2_Option			;  Option Button Pressed

  movlw	 0x01C			;  Is the Button Held Down for 20 msecs?  
  movwf	 TempHi
  movlw	 0x0E7

Menu2_OptionLoop		;  Loop Here for 20 msecs

  btfsc	 PORTA, 1		;  Is the Button Still Pressed?
   goto	 Menu2_Loop		;   No - Go Back to the Other Loop

  addlw	 0x0FF
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu2_OptionLoop

  incf	 SRAMAddr		;  Increment the SRAM Address 

  btfss	 SRAMAddr, 7		;  Have we changed from Disabled?
   goto	 Menu2_OptionLow	;  Check for Something Below 0x0100

  movlw	 0x064			;  Set the SRAMAddr to 0x0C8000
  movwf	 SRAMAddr

  goto	 Menu2_NewAddr

Menu2_OptionLow			;  Check to See if Address Between 0x0C8000 and 0x00DE000

  movf	 SRAMAddr, w
  xorlw	 0x070			;  At 0x0E0000?
  btfss	 STATUS, Z		
   goto	 Menu2_NewAddr		;  The Address is Valid

  movlw	 0x080			;  Set the Memory to Disabled
  movwf	 SRAMAddr

Menu2_NewAddr			;  The Address in "SRAMAddr" is Valid

  movf	 SRAMAddr, w
  movwf	 PORTC

  goto	 Menu2_Addr		;  Return to the Caller


Menu3				;  Enable/Disable the SRAM Writes

  movlw	 0x0FE			;  Clear the Display
  call	 SendChar
  movlw	 1			
  call	 SendChar

  movlw	 'S'			;  Output "SRAM Address" to the LCD
  call	 SendChar
  movlw	 'R'
  call	 SendChar
  movlw	 'A'
  call	 SendChar
  movlw	 'M'
  call	 SendChar
  movlw	 ' '
  call	 SendChar
  movlw	 'W'
  call	 SendChar
  movlw	 'r'
  call	 SendChar
  movlw	 'i'
  call	 SendChar
  movlw	 't'
  call	 SendChar
  movlw	 'e'
  call	 SendChar
  movlw	 's'
  call	 SendChar

Menu3_Value			;  Display the SRAM Write Enable

  movlw	 0x0FE			
  call	 SendChar
  movlw	 0x0C0
  call	 SendChar

  btfss	 PORTA, 3		;  Is the SRAM Writing Disabled?
   goto	 Menu3_Enabled

  movlw	 'D'			;  Yes, Note it
  call	 SendChar
  movlw	 'i'
  call	 SendChar
  movlw	 's'
  call	 SendChar
  movlw	 'a'
  call	 SendChar
  movlw	 'b'
  call	 SendChar
  movlw	 'l'
  call	 SendChar
  movlw	 'e'
  call	 SendChar
  movlw	 'd'
  call	 SendChar

  goto	 Menu3_KeyUp

Menu3_Enabled			;  SRAM Writing is Enabled

  movlw	 'E'			
  call	 SendChar
  movlw	 'n'
  call	 SendChar
  movlw	 'a'
  call	 SendChar
  movlw	 'b'
  call	 SendChar
  movlw	 'l'
  call	 SendChar
  movlw	 'e'
  call	 SendChar
  movlw	 'd'
  call	 SendChar
  movlw	 ' '
  call	 SendChar

Menu3_KeyUp			;  Wait for the Keys to be Released

  movf	 PORTA, w
  andlw	 0x006			;  If Button 1 or 2 is Pressed, Keep Looping
  xorlw	 0x006
  btfss	 STATUS, Z
   goto	 Menu3_KeyUp

  movlw	 0x0C2			;  have to wait 20 msecs for Release
  movwf	 Temp
  movlw	 0x012
  movwf	 TempHi

Menu3_KeyUpWait			;  Wait for the Button to be Released

  movf	 PORTA, w		;  Are the Buttons Still Down?
  andlw	 0x006
  xorlw	 0x006
  btfss	 STATUS, Z
   goto	 Menu3_KeyUp		;  Nope, Loop Around

  movlw	 0x0FF			;  At the End of the Delay?  
  addwf	 Temp
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu3_KeyUpWait

Menu3_Loop:			;  Now, Loop Forever reading the port

  btfss	 PORTA, 1		;  Check the Option Button
   goto	 Menu3_Option

  btfss	 PORTA, 2		;  Check the Menu Button
   goto	 Menu3_Menu

  goto	 Menu3_Loop		;  Loop Around Again, Polling Buttons

Menu3_Menu			;  Menu Button Pressed

  movlw	 0x01C			;  Is the Button Held Down for 20 msecs?  
  movwf	 TempHi
  movlw	 0x0E7

Menu3_MenuLoop			;  Loop Here for 20 msecs

  btfsc	 PORTA, 2		;  Is the Button Still Pressed?
   goto	 Menu3_Loop		;   No - Go Back to the Other Loop

  addlw	 0x0FF
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu3_MenuLoop

  movlw	 1			;  Note that the First Menu is to Execute
  movwf	 Menu

  goto	 Menu_Select		;  Jump to the Menu Selection

Menu3_Option			;  Option Button Pressed

  movlw	 0x01C			;  Is the Button Held Down for 20 msecs?  
  movwf	 TempHi
  movlw	 0x0E7

Menu3_OptionLoop		;  Loop Here for 20 msecs

  btfsc	 PORTA, 1		;  Is the Button Still Pressed?
   goto	 Menu3_Loop		;   No - Go Back to the Other Loop

  addlw	 0x0FF
  btfsc	 STATUS, Z
   decfsz TempHi
    goto Menu3_OptionLoop

  movlw	 8			;  Flip the Bit to be Dispalyed
  xorwf	 PORTA

  goto	 Menu3_Value		;  Return to the Caller


;  Subroutines
SendHex				;  Send the Hex of the Character in "w"

  movwf	 Temp

  swapf	 Temp, w
  andlw	 0x00F			;  Display the High Nybble First
  addlw	 0x036	            	;  Add '0' + 6 to Value
  btfsc  STATUS, DC       	;  If Digit Carry Set, then 'A' - 'F'
   addlw 7                	;   Add Difference Between '9' and 'A'  addlw  0-6
  addlw  0-6			;  Restore to Correct Size
  call	 SendChar

  movf	 Temp, w
  andlw	 0x00F			;  Display the High Nybble First
  addlw	 0x036	            	;  Add '0' + 6 to Value
  btfsc  STATUS, DC       	;  If Digit Carry Set, then 'A' - 'F'
   addlw 7                	;   Add Difference Between '9' and 'A'  addlw  0-6
  addlw  0-6			;  Restore to Correct Size
  call	 SendChar

  return

SendChar			;  Send a Character at 19,200 bps (52 usecs/bit)
				;  NOTE!  Data is Reversed
  movwf	 CharOut		;  Save the Character to Send

  movlw	 15			;  Output 10 Bits with Some Stops for the SLI-OEM
  movwf	 Count

  bcf	 STATUS, C		;  Use the Status Bit as the Next Character to Send

SC_Loop				;  Loop Here for Each Character

  btfsc	 STATUS, C		;  Send a Low Bit?  
   goto	 SC_High		;  Nope

  nop				;  Delay One Cycle

  bsf	 PORTA, 0		;  Output the Zero

  goto	 SC_Skip		;  Now, Go to the Next Bit

SC_High				;  Output the High Bit

  bcf	 PORTA, 0

  goto	 SC_Skip

SC_Skip

  movlw	 29 			;  Delay 41 Cycles for the Loop
SC_Dlay
  addlw	 0x0FF
  btfss	 STATUS, Z
   goto	 SC_Dlay
  nop

  goto	 $+1			;  Make Sure Cycles Line Up for 52 usecs

  bsf	 STATUS, C		;  Shift to the Right the Character 
  rrf	 CharOut		;   and Load the Carry Flag

  decfsz Count			;  Decrement the Counter
   goto	 SC_Loop		;   And if Not Equal to Zero, Send the Next Char

  return

LongDlay			;  Dlay 80 msecs

  clrf	 Count
  clrf	 Counthi

LD_Loop
  decfsz Count
   goto	 LD_Loop
  decfsz Counthi
   goto	 LD_Loop

  return

  end
