 title  "JoyStC - Wait for Characters to Be Sent."
;
;  This Application is a follow on to "JoystB" and Reads/Displays Eight
;   Bit Bytes Sent from a PC Host.  
;
;  To Send a "1", Two Pulses are Sent Back to Back, the "low" periode 
;   between the pulses being active for a minimal mount of time Possible.   
;  Receiving a "0" has a "low" Period of more than 50 usecs between the 
;   two Pulses (but Less than 255 usecs)
;
;  Each Byte is Eight bits Long and the Most Significant Bit is Sent 
;   First.  At the End of the String, a "Parity" Bit is Sent with a 
;   Pulse which is Followed With the PICMicro Pulling it Low if the 
;   Parity is Correct.  
;
;
;  Hardware Notes:
;   The Pic is Running with a 4 MHz Ceramic Oscillator
;   PortB is used for the LCD Data Port
;   PORTA.0 is the LCD's "R/W" Pin
;   PORTA.1 is the LCD's "R/S" Pin
;   PORTA.2 is the LCD's "E" Pin
;   PORTA.3 is the Joystick "Analog 1" (Joystick "X") Input Port
;   PortA.4 is the Joystick "Button 1" (Joystick Button 1) Output
;
;
;  Myke Predko
;  99.01.25
;
  LIST P=16C84, R=DEC           ;  PIC Runs at 4 MHz
  errorlevel 0,-305
  INCLUDE "p16c84.inc"


;  File Register Usage
 CBLOCK 0x00C
Dlay                            ;  8 Bit Delay Value
Temp				;  Temporary Output Value
CharIn:2			;  Character Read In
BitCount			;  Bit Counter
Parity				;  Parity Value
 ENDC

;  Flags Definition

;  Bit Definition
#define E_Pin  PORTA,2
#define RS_Pin PORTA,1
#define RW_Pin PORTA,0

;  Macros
EStrobe MACRO                   ;  Strobe the "E" Pin
  bsf    E_Pin
  bcf    E_Pin
 ENDM


 PAGE
 __CONFIG _CP_OFF & _PWRTE_ON  & _WDT_OFF & _XT_OSC
                                ;  Note that the WatchDog Timer is OFF
;  Code for JoyStick

  org    0

  movlw	 0x010			;  Drive RA4 Normally High
  movwf  PORTA                  
  clrf   PORTB


  bsf    STATUS, RP0            ;  Enable the PortB Output Drivers
  movlw  0x008                  ;  Enable the PORTA LCD Outputs
  movwf  TRISA ^ 0x080
  clrf   TRISB ^ 0x080
  movlw	 0x0D0			;  Enable the Timer on the Internal Clock
  movwf	 OPTION_REG ^ 0x080
  bcf    STATUS, RP0

#define tDebug

 ifdef Debug

  nop
  nop
  nop
  nop

  nop
  nop
  EStrobe
  nop

  EStrobe			;  Write Initial Reset Again to LCD
  nop

  EStrobe			;  Write Initial Reset Again to LCD
  nop

  nop
  nop

  nop
  nop

  nop
  nop

  nop
  nop

 else				;  Not Debug, Initialize the LCD

  call   Wait5                  ;  Wait 20 msecs before Sending Init Code
  call   Wait5
  call   Wait5
  call   Wait5

  movlw  0x030                  ;  Write the Initial Reset to LCD
  movwf  PORTB
  EStrobe
  call   Wait5

  EStrobe			;  Write Initial Reset Again to LCD
  call	 Wait160

  EStrobe			;  Write Initial Reset Again to LCD
  call	 Wait160

  movlw	 0x038			;  Write to Set the 2 Line Display
  call	 SendINS

  movlw	 0x00C			;  Turn on the Display
  call	 SendINS

  movlw	 0x001			;  Clear the Display
  call	 SendINS

  movlw	 'C'			;  Indicate joyst"C"
  call	 SendCHAR

 endif				;  End Joystick Initialization

Reset_Loop:			;  Come Back Here for Missing Bit

  bsf	 PORTA, 4		;  Make Sure there is a "NAK" Output

  btfsc	 PORTA, 3		;  Wait for the Port to Go Low
   goto	 $ - 1

Loop                            ;  Now, Loop Forever/Monitoring RA3

  movlw	 9			;  Initialize the Bit Counter
  movwf	 BitCount

  clrf	 CharIn			;  Clear the Character Read
  clrf	 CharIn + 1

  clrf	 Parity			;  Clear the Parity

  btfss	 PORTA, 3		;  Wait for Port3 to Pulse High/Low
   goto	 $ - 1

Bit_Loop			;  Come Back Here for Each Bit 

  btfsc	 PORTA, 3
   goto	 $ - 1

  clrf	 TMR0			;  Use the Timer for the Pulse Width

Time_Loop

  btfsc	 PORTA, 3		;  Wait for the Pulse to Go High
   goto	 Skip

  comf	 TMR0, w		;  Has the Timer Overflowed?
  btfsc	 STATUS, Z
   goto	 Reset_Loop		;   - Yes, Loop Around

  goto	 Time_Loop		;  Keep Waiting for the "High"

Skip				;  Have the High, Save the Value and Wait for Low

  movf	 TMR0, w

  btfsc	 PORTA, 3
   goto	 $ - 1

;  addlw	 0 - 50			;  Are we Less than or Greater than 50 usecs?
  addlw  0 - 25			;  Get the Actual 50 msecs (which is 25 on the Counter)
  btfss	 STATUS, C		;  Keep Track of the Parity
   incf	 Parity
  rlf	 CharIn			;  Load the Carry As the Next Bit
  rlf	 CharIn + 1

  clrf	 TMR0			;  Use the Timer for Bit Timeout Check

Time_Loop2

  btfsc	 PORTA, 3		;  Wait for the Pulse to Go High
   goto	 Skip2

  comf	 TMR0, w		;  Has the Timer Overflowed?
  btfsc	 STATUS, Z
   goto	 Reset_Loop		;   - Yes, Loop Around

  goto	 Time_Loop2		;  Keep Waiting for the "High"

Skip2				;  Have the High, Save the Value and Wait for Low

  decfsz BitCount		;  Decrement the Bit Counter
   goto	 Bit_Loop

  movf	 Parity, w		;  Compare the Parity Bits
  xorwf	 CharIn, w
  andlw	 1			;  Just Interested in the Last Bit
  btfss	 STATUS, Z
   goto	 Loop			;  Parity Mis-Match, Ignore the Character

  bcf	 PORTA, 4		;  Output the Acknowledge Bit

  btfsc	 PORTA, 3		;  Wait for the Bit to Go Low
   goto	 $ - 1

  bsf	 PORTA, 4		;  Turn off the Ack Bit

  rrf	 CharIn + 1		;  Display the Character
  rrf	 CharIn, w
  xorlw	 0x0FF			;  Have to Read In and Invert
  call	 SendCHAR		;  Output the Bit Read

  goto   Loop


;  Subroutines
SendINS				;  Send an Instruction to the LCD

  bcf	 RS_Pin

  movwf	 PORTB			;  Save the Value in "w"
  EStrobe

  andlw	 0x0FC			;  Wait 160 usecs of 5 msec?
  btfsc	 STATUS, Z
   goto	 SendINS_Long

  call	 Wait160

  goto	 SendINS_End

SendINS_Long

  call	 Wait5

SendINS_End

  return
  
SendCHAR			;  Send a Character to the LCD

  bsf	 RS_Pin

  movwf	 PORTB			;  Save the Value in "w"
  EStrobe

  call	 Wait160

  return
  

Wait5                           ;  Wait 5 milliseconds

  movlw  0x004                  ;  High Byte of Delay Value
  movwf  Dlay
  movlw  0x018

Wait5_Loop                      ;  5 msecs per loop
  addlw  1
  btfsc  STATUS, C
   decfsz Dlay
    goto Wait5_Loop

  return

Wait160				;  Wait 160 usecs for the Command to Complete

  movlw  0x001                  ;  High Byte of Delay Value
  movwf  Dlay
  movlw  0x0E0

Wait160_Loop                    ;  160 usecs per loop
  addlw  1
  btfsc  STATUS, C
   decfsz Dlay
    goto Wait160_Loop

  return

  end
