//  JoyStC - Send Characters through thee Joystick Port
//
//  This Program sends "1"s and "0"s to the Joystick Port
//   Using the Protocol:
//
//        __          __
//   ____|  |________|  |_______
//          ^        ^
//          |        |
//           Data Bit
//
//  Where a "Data Bit" > 50 usec (But Less than 255 usec) is a "0"
//   and a "Data Bit" < 50 usec is a "1"
//
//  Nine Bits are sent for Each Byte with the Last Bit being an
//   "Odd" Parity Bit.
//
//  At the End of the Byte, the PC Pulses the Joystick Line and
//   Looks to see if the Microcontroller "Acks" (Pulls Down
//   Button 1).
//
//  Myke Predko
//  99.01.25
//


//  Defines
#include <stdio.h>              //  For "printf"
#include <dos.h>                //  For "inp"/"outp"
#include <asmprocs.h>           //  For "GETKEY"/"GETKEYSTAT"


//  Global Variables


//  External Subroutines
DLAY( int Value );              //  Delay the Set Number of
                                //   838 nsec Cycles


//  Subroutines
BitPulse( int Value )
{

int  i;


  outp( 0x0201, 0 );            //  Send out the Pulse
  while (( inp( 0x0201 ) & 1 ) != 0 );

  for ( i = 2 + (( Value ^ 1 ) * 98 ); i > 0; i-- )
    inp( 0x0300 );              //  Put in ISA "Read" Delays

  outp( 0x0201, 0 );            //  Send the Second Pulse
  while (( inp( 0x0201 ) & 1 ) != 0 );

  for ( i = 0; i < 5; i++ )
    inp( 0x0300 );              //  Let the MCU "Settle"

}  //  End BitPulse


int ByteSend( char DataOut )    //  Send the Byte Out
{

int  i;
int  Parity = 0;

  for ( i = 0; i < 8; i++ ) {   //  Send the Data a Bit at a Time
    if (( DataOut & 0x080 ) == 0 )
      Parity++;                 //  Keep Track of the Parity
    BitPulse(( DataOut >> 7 ) & 1 );
    DataOut = ( DataOut << 1 ) & 0x0FF;
  }  /*  endfor  */

  BitPulse(( Parity & 1 ) ^ 1 );  //  Send the Parity

  outp( 0x0201, 0 );  i = 0;    //  Now, Look for the Ack
  while (( inp( 0x0201 ) & 1 ) != 0 )
    if (( inp( 0x0201 ) & 0x010 ) == 0 )
      i = 1;

  return i ^ 1;                 //  Return "0" For Good

}  // End ByteSend


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

int  Key = 0;                   //  KeyStroke Read In


  outp( 0x0201, 0 );            //  Make Sure ADC Lines Low

  printf( "JoyStC - Test Bit Transmit/Receive\n\n" );
  printf( "Myke Predko\n" );
  printf( "99.02.06\n\n" );

  printf( "Press a key to Send a Bit\n" );
  printf( "Press \"Enter\" to End\n" );


  while ( GETKEYSTAT() != 0 )   //  Clear the Keyboard Buffer
    GETKEY();

  while ((( Key = GETKEY() ) & 0x0FF ) != 0x0D )
    if ( ByteSend( Key & 0x0FF ) != 0 )
      printf( "No Ack Received from the Interface\n" );

}  //  End main
