 title  "Key_MonD - Display the Keyboard Characters."
;
;  This Application Displays what is on the Keyboard Lines and Displays
;   the Hex Codes on an LCD.
;
;  Key_MonD Displays Keys from the System and the Keyboard and Displays
;   it as Hex.
;
;
;  Hardware Notes:
;   The Pic is Running with a 4 MHz Ceramic Oscillator
;   PortB is used for the LCD Data Port
;   PORTA.0 is the LCD's "R/W" Pin
;   PORTA.1 is the LCD's "R/S" Pin
;   PORTA.2 is the LCD's "E" Pin
;   PORTA.3 is the Keyboard Data
;   PortA.4 is the Keyboard Clock
;
;
;  Myke Predko
;  99.02.04
;
  LIST P=16C84, R=DEC           ;  PIC Runs at 4 MHz
  errorlevel 0,-305
  INCLUDE "p16c84.inc"


;  File Register Usage
 CBLOCK 0x00C
Dlay                            ;  8 Bit Delay Value
Temp                            ;  Temporary Output Value
CharIn:2                        ;  Character Read In
BitCount                        ;  Bit Counter
Parity                          ;  Parity Value
 ENDC

;  Flags Definition

;  Bit Definition
#define E_Pin  PORTA,2
#define RS_Pin PORTA,1
#define RW_Pin PORTA,0

;  Macros
EStrobe MACRO                   ;  Strobe the "E" Pin
  bsf    E_Pin
  bcf    E_Pin
 ENDM


 PAGE
 __CONFIG _CP_OFF & _PWRTE_ON  & _WDT_OFF & _XT_OSC
                                ;  Note that the WatchDog Timer is OFF
;  Code for Keyboard Monitor

  org    0

  clrf   PORTA
  clrf   PORTB

  bsf    STATUS, RP0            ;  Enable the PortB Output Drivers
  movlw  0x018                  ;  Enable the PORTA LCD Outputs
  movwf  TRISA ^ 0x080
  clrf   TRISB ^ 0x080
  movlw  0x0D0                  ;  Enable the Timer on the Internal Clock
  movwf  OPTION_REG ^ 0x080
  bcf    STATUS, RP0


  call   Wait5                  ;  Wait 20 msecs before Sending Init Code
  call   Wait5
  call   Wait5
  call   Wait5

  movlw  0x030                  ;  Write the Initial Reset to LCD
  movwf  PORTB
  EStrobe
  call   Wait5

  EStrobe                       ;  Write Initial Reset Again to LCD
  call   Wait160

  EStrobe                       ;  Write Initial Reset Again to LCD
  call   Wait160

  movlw  0x038                  ;  Write to Set the 2 Line Display
  call   SendINS

  movlw  0x00C                  ;  Turn on the Display
  call   SendINS

  movlw  0x001                  ;  Clear the Display
  call   SendINS


Reset_Loop:                     ;  Come Back Here for Missing Bit

  movf   PORTA, w               ;  Look for the Incoming Characters
  andlw  0x018
  xorlw  0x018
  btfsc  STATUS, Z
   goto  Reset_Loop             ;  Nothing Coming In

  btfsc  PORTA, 3               ;  Check for the Clock Low
   goto  PC_Data                ;   PC Data Coming In

;  Read in the Keyboard Data

  movlw  9                      ;  Read in the Keyboard Data
  movwf  BitCount               ;  go to the First 9 Characters

Key_Loop                        ;  Read in the First 9 Bits

  btfsc  PORTA, 4               ;   Wait for the Clock to Go Low
   goto  $ - 1

  bcf    STATUS, C              ;  What is the bit Coming in?
  btfsc  PORTA, 3
   bsf   STATUS, C              ;  High Bit
  rrf    Temp                   ;  Shift in the Bit

  btfss  PORTA, 4               ;  Wait for Bit to Go High
   goto  $ - 1

  decfsz BitCount
   goto  Key_Loop

  btfsc  PORTA, 4               ;  Wait for the Parity Bit
   goto  $ - 1

  btfss  PORTA, 4
   goto  $ - 1

  btfsc  PORTA, 4               ;  Wait for LOOONG Stop Bit
   goto  $ - 1

  movlw  'k'                    ;  Identify the incoming Byte
  call   SendCHAR

  movf   Temp, w                ;  Display the Byte
  call   SendHEX

  btfss  PORTA, 4               ;  Wait for Bit End
   goto  $ - 1

  goto   Reset_Loop


;  Read in the PC's Data
PC_Data

  movlw  9                      ;  Read in the Keyboard Data
  movwf  BitCount               ;  go to the First 9 Characters

  btfsc  PORTA, 4               ;  Wait for the Clock to Go Low
   goto  $ - 1

PC_Loop                         ;  Read in the First 9 Bits

  btfss  PORTA, 4               ;   Wait for the Clock to Go Low
   goto  $ - 1

  bcf    STATUS, C              ;  What is the bit Coming in?
  btfsc  PORTA, 3
   bsf   STATUS, C              ;  High Bit
  rrf    Temp                   ;  Shift in the Bit

  btfsc  PORTA, 4               ;  Wait for Bit to Go High
   goto  $ - 1

  decfsz BitCount
   goto  PC_Loop

  btfss  PORTA, 4               ;  Wait for the Parity Bit
   goto  $ - 1

  btfsc  PORTA, 4
   goto  $ - 1

  btfss  PORTA, 4               ;  Wait for LOOONG Stop Bit
   goto  $ - 1

  movlw  'P'                    ;  Identify the Incoming Byte
  call   SendCHAR

  movf   Temp, w                ;  Display the Byte
  call   SendHEX

  btfsc  PORTA, 4               ;  Wait for Bit End
   goto  $ - 1

  goto   Reset_Loop


;  Subroutines
SendINS                         ;  Send an Instruction to the LCD

  bcf    RS_Pin

  movwf  PORTB                  ;  Save the Value in "w"
  EStrobe

  andlw  0x0FC                  ;  Wait 160 usecs of 5 msec?
  btfsc  STATUS, Z
   goto  SendINS_Long

  call   Wait160

  goto   SendINS_End

SendINS_Long

  call   Wait5

SendINS_End

  return

SendHEX                         ;  Send a Hex Character to the LCD

  movwf  Temp                   ;  Save the Character

  swapf  Temp, w                ;  Do the Hi Nybble First
  andlw  0x00F
  addlw  0x036                  ;  My Nybble to ASCII code
  btfsc  STATUS, DC
   addlw 7
  addlw  0-6
  call   SendCHAR

  movf   Temp, w                ;  Do the Lo Nybble Second
  andlw  0x00F
  addlw  0x036                  ;  My Nybble to ASCII code
  btfsc  STATUS, DC
   addlw 7
  addlw  0-6
  call   SendCHAR

  return

SendCHAR                        ;  Send a Character to the LCD

  bsf    RS_Pin

  movwf  PORTB                  ;  Save the Value in "w"
  EStrobe

  call   Wait160

  return


Wait5                           ;  Wait 5 milliseconds

  movlw  0x004                  ;  High Byte of Delay Value
  movwf  Dlay
  movlw  0x018

Wait5_Loop                      ;  5 msecs per loop
  addlw  1
  btfsc  STATUS, C
   decfsz Dlay
    goto Wait5_Loop

  return

Wait160                         ;  Wait 160 usecs for the Command to Complete

  movlw  0x001                  ;  High Byte of Delay Value
  movwf  Dlay
  movlw  0x0E0

Wait160_Loop                    ;  160 usecs per loop
  addlw  1
  btfsc  STATUS, C
   decfsz Dlay
    goto Wait160_Loop

  return

  end
