PAGE ,132                               ;  - Put Printer in 132 Character Mode
TITLE Load.ASM - Display Boot Information
;
;       This Program is a simple boot sector which Loads the first 8K of
;       a .com program Starting at Sector 2 of the Boot Disk into memory
;       and starts executing at offset 0x0100.
;
;       The Segment is Chosen as the start Address + 256.  The Code is loaded
;       at Start Address + 512.  Before the Code Starts, the registers are set
;       up in the following order:
;
;       AX, BX, CX, DX = 0
;       DI, SI, BP = 0
;       DS, ES, SS = New Segment
;       CS:IP = New Segment:0x0100
;
;       Myke Predko
;       Started: 98.11.18
;
PAGE
;
;       CONVENTIONS
;
;       CAPITALS  - CONSTANTS, ASSEMBLER KEYWORDS
;       MixedCase - Variables, Procedure Names, Macros, Structures
;       lowercase - assembler mnemonics
;
;
;       CHAPTERS
;
;       Chapter  1 - Program Information
;       Chapter  2 - Publics and Externals
;       Chapter  3 - Program Segment Information
;       Chapter  4 - Constants
;       Chapter  5 - Structures
;       Chapter  6 - Macros
;       Chapter  7 - Mainline
;       Chapter  8 - Subroutines
;       Chapter  9 - Variables
;


PAGE+
;  Define Publics for Subroutines in the Member


PAGE+
ALLPROCS SEGMENT                ;  Segment for Everything
        ASSUME  CS:ALLPROCS, DS:ALLPROCS, ES:ALLPROCS, SS:ALLPROCS


PAGE+
;  Constants
DISKINT  EQU    013h            ;  Diskette Interrupt
DISKRST  EQU    000h            ;  Reset Disk Sub-System
DISKREAD EQU    002h            ;  Read the Diskette

VIDINT   EQU    010h            ;  BIOS Video Interrupt
READCURS EQU    003h            ;  Read the Cursor Position
MOVECURS EQU    002h            ;  Set the Cursor Position
VIDWRITE EQU    009h            ;  Write Attribute/Byte at Cursor


PAGE+
;  Structures
PtrFmt  STRUC                   ;  Pointer Structure Format
PtrOff  dw      ?               ;  Offset
PtrSeg  dw      ?               ;  Segment
PtrFmt  ENDS


PAGE+
;  Macros


PAGE+
;  LOAD - Load the Application into the PC's Memory
 org 0                          ;  Start at Address 0
START:

  call  GetAddr                 ;  Get the Actual Starting Address
IPValue:                        ;  Actual IP Value
  mov   AX, SI
  lea   BX, IPValue
  sub   AX, BX
  add   AX, 00100h              ;  Move 512 Bytes Up
  mov   ES, AX                  ;  Save the Segment Start for Later
  shr   AX, 1                   ;  Shift down by 4 to Get the Correct Paragraph
  shr   AX, 1
  shr   AX, 1
  shr   AX, 1
  mov   BX, CS
  add   AX, BX                  ;  Make Sure Correct for Debugging
  mov   DS, AX
  mov   SS, AX
  add   AX, 010h                ;  Point to the Save Address for the Read
  mov   ES, AX
  mov   SP, 0FFFEh              ;  Setup the Stack Pointer

  mov   AH, DISKRST             ;  Reset the Disk Sub-System
  mov   DL, 0
  int   DISKINT

  mov   AH, DISKREAD            ;  READ the Application from the Diskette
  mov   DL, 0
  mov   DH, 0                   ;  Head 0
  mov   CH, 0                   ;  Track 0
  mov   CL, 2                   ;  Sector 2 (After Boot)
  mov   AL, 17                  ;  Read in .COM's First 8704 Bytes
  mov   BX, 0000h               ;  Point Past Currently Loaded Code
  int   DISKINT

  call  GetAddr
  add   SI, JumpAddr - $        ;  Update the Jump Address
  mov   WORD PTR CS:[SI], 0100h
  mov   WORD PTR CS:[SI + 2], DS

  xor   AX, AX                  ;  Clear all the Registers
  xor   BX, BX
  xor   CX, CX
  xor   DX, DX

  xor   DI, DI
  xor   SI, SI
  xor   BP, BP

  push  DS                      ;  Setup ES
  pop   ES

  db    0EAh                    ;  Jump to the Code
JumpAddr:
  dd    0                       ;  Address for the Instruction


PAGE+
;  Subroutines
GetAddr:                        ;  Get the Address of the Call

  pop   SI

  push  SI

  ret


PAGE+
;  Variables

ALLPROCS ENDS
        END     START