// RegSampl - Do a Register Sample/Output for the SoundBlaster
//
//  This program is a "C" rewrite of sample assembly code found on
//   the web.  This application does a Sample Read of the Input ADC
//   line and then saves it in an array.  When the Load is Complete,
//   the array is output through the speaker ports.
//
//  Myke Predko
//  99.02.22
//

//  Defines
#include <stdio.h>
#include <asmprocs.h>


//  Global Variables
char far * Buffer;              //  buffer of the Read in Data


//  Subroutines
WriteDSP( int Value )           //  Wait for the Port to be free and
{                               //   then Write a Command


  while (( inp( 0x022C ) & 0x080 ) != 0 );

  outp( 0x022C, Value );

}  // WriteDSP

int ReadDSP()                   //  Wait for the ADC Operation to
{                               //   Complete


  while (( inp( 0x022E ) & 0x080 ) == 0 );

  return inp( 0x022A );		  //  Return the DSP Data When Ready

}  //  End ReadDSP


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

int i, j;


  Buffer = GETMEM( 0x08000L );

  printf( "RegSampl - Myke Predko\n" );
  printf( "           99.02.22\n\n" );

  printf( "Speak into the Microphone, Press Any Key when Done\n\n" );

  while ( GETKEYSTAT() != 0 )
    GETKEY();                   //  Clear the Keyboard Buffer

  printf( "Recording" );

  for ( i = 0; ( i < 0x08000 ) && ( GETKEYSTAT() == 0 ); i++ ) {

    if (( i & 0x01FF ) == 0 )
      printf( "." );            //  Printing 64 Dots

    WriteDSP( 0x020 );          //  Do a Direct Read
    Buffer[ i ] = ReadDSP();    //  Wait for the DMA to Complete

  }  //  endfor

  printf( "\n" );               //  Star the Next Line

  if ( GETKEYSTAT() == 0 ) {    //  Only Output if No Key Pressed

    printf( "\nPlayback" );

    WriteDSP( 0x0D1 );          //  Enable the Speaker for Output

    for ( i = 0; ( i < 0x08000 ) && ( GETKEYSTAT() == 0 ); i++ ) {

      if (( i & 0x01FF ) == 0 )
        printf( "." );          //  Printing 64 Dots

      WriteDSP( 0x010 );        //  Output DAC Data Byte
      WriteDSP( Buffer[ i ]);
      ReadDSP();

    }  //  endfor

    WriteDSP( 0x0D3 );          //  Turn Off the Speaker

  }  //  endif

  while ( GETKEYSTAT() != 0 )   //  Clear the Keyboard Buffer
    GETKEY();                   //  If Keys Used to End Operation

  FREEMEM( Buffer );

}  //  End RegSampl