//  WAVEDMA - Application to Play "Wave file" data on a PC using DMA
//
//  This program loads the specified ".WAV" file and plays it through
//   the PC's soundBlaster card using the DMA controller to pass the
//   data at a specific sampling rate (specified within the .WAV
//   file).
//
//  Myke Predko
//  99.02.23
//

//  Defines
#include <stdio.h>
#include <dos.h>
#include <asmprocs.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <string.h>

#define DMA0Addr  0x000         //  DMA Address Register
#define DMA0Word  0x001         //  DMA Word count Register
#define DMA1Addr  0x002
#define DMA1Word  0x003
#define DMA2Addr  0x004
#define DMA2Word  0x005
#define DMA3Addr  0x006
#define DMA3Word  0x007
#define DMA1Stat  0x008         //  DMA Status/Command Register
#define DMA1Write 0x009         //  DMA Write Request Register
#define DMA1Mask  0x00A         //  DMA Mask Register
#define DMA1Chnl  0x00B         //  DMA Channel Mode Register
#define DMA1Clr   0x00C         //  DMA Clear Byte Pointer
#define DMA1Mstr  0x00D         //  DMA MasterRegister
#define DMA1CMsk  0x00E         //  DMA Clear Mask Register
#define DMA1WMsk  0x00F         //  DMA Write Mask Register
#define DMA2Page  0x081         //  DMA Channel 2 Page Address
#define DMA3Page  0x082         //  DMA Channel 3 Page Address
#define DMA1Page  0x083         //  DMA Channel 1 Page Address


//  Global Variables
char far * Buffer;              //  For Loading the File Into
char FileName[ 256 ];           //  Plan for the Worst Possible Case


//  Subroutines
WriteDSP( int Value )           //  Wait for the Port to be free and
{                               //   then Write a Command


  while (( inp( 0x022C ) & 0x080 ) != 0 );

  outp( 0x022C, Value );

}  // WriteDSP

int ReadDSP()                   //  Wait for the ADC Operation to
{                               //   Complete


  while (( inp( 0x022E ) & 0x080 ) == 0 );

  return inp( 0x022A );         //  Return the DSP Data When Ready

}  //  End ReadDSP

int ResetDSP()                  //  Initialize the SoundBlaster
{

int  i;
int  retvalue = -1;


  outp( 0x0226, 0x001 );        //  Start Reset Operation
  for ( i = 0; i < 5; i++ )
    inp( 0x0226 );
  outp( 0x0226, 0x000 );        //  Complete the Reset

  for ( i = 0; ( i < 256 ) && ( retvalue == - 1 ); i++ )
    if ( inp( 0x022A ) == 0x0AA )
      retvalue = 0;             //  Reset Successful

  return retvalue;

}  //  End ResetDSP

SBSetup()                       //  Look for Sound Blaster Version
{                               //   3.x or Later for Mixer

int VerMajor, VerMinor;


  WriteDSP( 0x0E1 );            //  Get Version Information
  VerMajor = ReadDSP();
  VerMinor = ReadDSP();

  if ( VerMajor == 3 ) {        //  Setup Mixor if Version 3.x

    outp( 0x0224, 0x000 );  outp( 0x0225, 0x0FF );
    outp( 0x0224, 0x004 );  outp( 0x0225, 0x0FF );
    outp( 0x0224, 0x00A );  outp( 0x0225, 0x000 );
    outp( 0x0224, 0x00C );  outp( 0x0225, 0x026 );
    outp( 0x0224, 0x00E );  outp( 0x0225, 0x020 );
    outp( 0x0224, 0x022 );  outp( 0x0225, 0x099 );
    outp( 0x0224, 0x026 );  outp( 0x0225, 0x000 );
    outp( 0x0224, 0x028 );  outp( 0x0225, 0x000 );
    outp( 0x0224, 0x02E );  outp( 0x0225, 0x000 );

  }  /*  endif  */
}  // End SBSetup


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

int  i;
int  handle;
long size;
int  temp;
long addrtemp;

char far * ActBuffer;           //  Actual Location in Buffer
int  Samples;                   //  Samples per Second
int  DataSize;                  //  Number of Bytes to Send


  printf( "\"WAVEDMA\" - Play a .WAV File on the PC\n" );
  printf( "Myke Predko\n\n" );

  if ( argc != 2 )              //  File Specified?
    printf( "No .WAV File Specified\n" );
  else {                        //  Load the File/Read it

    strcpy( FileName, argv[ 1 ]);  //  Get Proper FileName
    for ( i = strlen( FileName ) - 1; ( i > 0 ) &&
         ( FileName[ i ] != '.' ) && ( FileName[ i ] != '\\' ); i-- );
    if ( FileName[ i ] != '.' )
      strcat( FileName, ".wav" );

    if (( handle = open( FileName, O_RDONLY | O_BINARY )) == - 1 )
      printf( "\"%s\" could not be found\n", FileName );
    else {                      //  Go Ahead and Process the File

      if (( size = lseek( handle, 0L, SEEK_END )) > 0x08000L )
        printf( "\"%s\" is Too Large for the Application\n", FileName );
      else                      //  Can Handle the Application
        if (( Buffer = GETMEM( 0x010000L )) == 0L )
          printf( "Unable to Allocate the Required Memory for Application\n" );
        else {                  //  Can Execute the Application

          lseek( handle, 0L, SEEK_SET );

          addrtemp = ( GETSEG( Buffer ) << 4 ) & 0x0FFFFL;
          if ( addrtemp > 0x08000 )
            (long) ActBuffer =
            (((long) GETSEG( Buffer ) & 0x0F000L ) + 0x01000L ) << 16;
          else                  //  Calculate Actual Buffer Address
            (long) ActBuffer =
            (((long) GETSEG( Buffer ) & 0x0F000L ) + 0x00800L ) << 16;

          read( handle, ActBuffer, size );  //  Read in the File

          if (( strncmp( &ActBuffer[ 0 ], "RIFF", 4 ) != 0 ) ||
                     ( strncmp( &ActBuffer[ 8 ], "WAVE", 4 ) != 0 ) ||
                    ( strncmp( &ActBuffer[ 12 ], "fmt ", 4 ) != 0 ) ||
                      ( strncmp( &ActBuffer[ 36 ], "data", 4 ) != 0 ))
            printf( "\"%s\" is not a Valid .WAV File\n", FileName );
          else                  //  Process the .WAV File
            if ( ResetDSP() != 0 )
              printf( "No SoundBlaster in PC\n" );
            else {              //  Go ahead and Process the File

              SBSetup();        //  Setup the SoundBlaster Output

              temp = ActBuffer[ 24 ] + ( ActBuffer[ 25 ] * 256 );
              Samples = 256 - ( 1000000L / temp );
              WriteDSP( 0x040 );  //  Setup the Sample Rate
              WriteDSP( Samples );

              WriteDSP( 0x0D1 );  //  Turn the Speaker On

              DataSize = ActBuffer[ 40 ] + ( ActBuffer[ 41 ] * 256 );
              DataSize--;       //  Have the Data to Send

              outp( DMA1Mask, 0x005 );  //  Mask Channel 1
              outp( DMA1Clr, 0x000 );   //  Clear
              outp( DMA1Chnl, 0x049 );  //  Set Mode
              outp( DMA1Addr, 0x000 );  //  Load Start Address
              if ( addrtemp > 0x08000 )
                outp( DMA1Addr, 0x000 );
              else
                outp( DMA1Addr, 0x080 );
              outp( DMA1Page, (unsigned long) ActBuffer >> 28 );
              outp( DMA1Word, DataSize & 0x0FF );
              outp( DMA1Word, ( DataSize >> 8 ) & 0x0FF );
              outp( DMA1Mask, 1 );      //  Unmask DMA

              WriteDSP( 0x014 );  //  Enable DSP DMA
              WriteDSP( DataSize & 0x0FF );
              WriteDSP(( DataSize >> 8 ) & 0x0FF );

              while (( inp( DMA1Stat ) & 2 ) == 0 );
                                //  Wait for DMA Output to End

              WriteDSP( 0x0D3 );  //  Turn off the Speaker

            }

          FREEMEM( Buffer );

        }  /*  endif  */

      close( handle );

    }
  }

}  //  End WAVEDMA