//  MEMORY.C - Find What has been Plugged into the PC
//
//  This Program is used to find unused memory in the PC, by looking
//   for RAM and ROM in the C8000 to DFFFF Memory Area.  Memory that
//   is found is identified and Displayed on the Screen.
//
//  Myke Predko
//
//  Started: 98.10.16
//  Updated: xx.xx.xx
//


//  Include Files
#include <stdio.h>
#include <dos.h>
#include <asmprocs.h>


//  Defines
#define KEYINT  0x016           //  Codes for Reading a Character
#define KEYREAD 0x000


//  Global Variables


//  External Routines


//  Subroutines
int MemCheck( unsigned long Addr, int far * Size )  // Look at Memory
{                               //   at Specified Addr/Determine Type

char far * TestAddr;            //  Pointer to the Test Address

long i;
int  j;
char Temp;                      //  Saved Value for RAM
long ROMSize;
int  retvalue = 0;              //  Assume Nothing There


  ( long ) TestAddr = Addr;

  if (( TestAddr[ 0 ] == 0x055 ) && (( TestAddr[ 1 ] & 0x0FF )
                                                         == 0x0AA )) {

    ROMSize = TestAddr[ 2 ] * 512L;  //  Look for ROM

    for ( i = j = 0; i < ROMSize; i++ )  //  Get CheckSum
      j += TestAddr[ i ];

    if (( j & 0x0FF ) == 0 ) {  //  Have Some ROM
      *Size = ROMSize / 512;
      retvalue = 2;
    }  //  endif
  }  //  endif

  if ( retvalue == 0 ) {        //  Check for RAM
    Temp = TestAddr[ 0 ];
    CLI();                      //  Disable Interrupts
    TestAddr[ 0 ] = TestAddr[ 0 ] ^ 0x0FF;
    if ((( Temp ^ 0x0FF ) & 0x0FF ) == TestAddr[ 0 ]) {
      TestAddr[ 0 ] = Temp;     //  If Change, put back
      STI();                    //  Turn Interrupts Back On
      retvalue = 1;
    }  //  endif
    STI();                      //  Turn Interrupts Back On
  }  //  endif

  return retvalue;

}  //  MemCheck


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

long i;
int  j, k;

union REGS regs;


  printf( "Memory Check:           +0x0400 +0x0800 +0x0C00 +0x1000 +0x1400 +0x1800 +0x1C00" );

  i = 0x0A0000L;                //  Start Reading
  while ( i < 0x0F0000L ) {     //  Loop to the End
    if (( i & 0x1FFFL ) == 0 )  //  Start New Line?
      printf( "\n0x0%5lX       ", i );
    if (( i & 0x03FFL ) == 0 )
      printf( "  " );           //  Space Out Columns
    switch ( MemCheck( i << 12, &j ))
      {
        case 1:                 //  RAM Found
          printf( "%c%c%c", 177, 177, 177 );
          i += 0x00200L;        //  Increment the Address
          break;
        case 2:                 //  ROM Found
          for ( k = 0; k < j; k++ ) {
            printf( "%c%c%c", 219, 219, 219 );
            i += 0x00200L;      //  Increment the Address
            if ((( i & 0x1FFFL ) == 0 ) && ( k < ( j - 1 )))
              printf( "\n0x0%5lX       ", i );
            if ((( i & 0x03FFL ) == 0 ) && ( k < ( j - 1 )))
              printf( "  " );   //  Space Out Columns
          }  //  endfor
          break;
        default:                //  Nothing Found, Skip Over
          printf( "   " );
          i += 0x00200L;        //  Increment the Address
      } /*  endswitch  */
  }  //  endwhile

  regs.h.ah = KEYREAD;          //  Wait for a Keystroke Before Ending
  int86( KEYINT, &regs, &regs );

  printf( "\n" );               //  Put in a New Line

}  //  end LIFE
