PAGE ,132                               ;  - Put Printer in 132 Character Mode
TITLE Delay.ASM - Delay ASM Routines - Version 1.00
;
;       Delay consists of some routines for providing relatively short
;       delays to PC Applications.
;
;       This Program MUST be Assembled and Linked in .EXE format (it has
;       it's own Code Segment).  This Program should be Linkable with all
;       IBM C/2 Formats, but will NOT be Linkable in a TINY (Same Code and
;       Data Segments) Format - this means this Routine CANNOT be linked
;       into .COM Format.
;
;       Myke Predko
;       98.11.24
;
;       Version 1.00 Contains:
;
;       int DLAY( int Delay )  - Delay has a granularity of
;                                 838 nsecs
;
PAGE
;
;       CONVENTIONS
;
;       CAPITALS  - CONSTANTS, ASSEMBLER KEYWORDS
;       MixedCase - Variables, Procedure Names, Macros, Structures
;       lowercase - assembler mnemonics
;
;
;       CHAPTERS
;
;       Chapter  1 - Program Information
;       Chapter  2 - Publics and Externals
;       Chapter  3 - Program Segment Information
;       Chapter  4 - Constants
;       Chapter  5 - Structures
;       Chapter  6 - Macros
;       Chapter  7 - "CAP" PreProccessor Subroutines
;       Chapter  8 - Variables
;


PAGE+
;  Define Publics for Subroutines in the Member
        PUBLIC  _DLAY           ;  Delay the Specified Value


PAGE+
PASMPROCS SEGMENT               ;  Program Code Segment...
        ASSUME  CS:PASMPROCS


PAGE+
;  Constants
GATE    EQU 061h                ;  Address of Timer Gate Register
TIMER   EQU 040h                ;  Address of 8254 Timer Chip


PAGE+
;  Structures
PtrFmt  STRUC                   ;  Pointer Structure Format
PtrOff  dw      ?               ;  Offset
PtrSeg  dw      ?               ;  Segment
PtrFmt  ENDS


PAGE+
;  Macros


PAGE+
;  _DLAY - Wait the Specified Number of "Ticks" (at 838 nsecs each)
;        - If Delay is > 100 usecs, the Routine Skips out
_DLAY   PROC    FAR
        ASSUME  CS:PASMPROCS
        push    bp              ;  Setup the Pointer to the Passed Parameters
        mov     bp, sp
        push    ax              ;  Save the Changed Registers
        push    bx
        push    cx
        push    dx
        mov     bx, [bp+6]      ;  Get the Number of "Ticks" to Delay

        mov     AX, -1          ;  Can This Safely work?
        cmp     bx, 119         ;   Not if there are more than 100 usecs
        jge     _DLAY_End

        in      AL, GATE        ;  Read the Current Gate Value
        mov     CL, AL          ;  Save the Value for After the PGM
        and     AL, 0FCh        ;  Make Sure the Gates are Off
        jmp     SHORT $+2       ;  Clear Prefetch Queue
        out     GATE, AL
        mov     CH, AL

        mov     AL, 0B6h        ;  Set the Timer to LSB/MSB Binary Mode
        out     TIMER + 3, AL
        jmp     SHORT $+2

        xor     AX, AX          ;  Reset the Timer
        out     TIMER + 2, AL
        jmp     SHORT $+2
        out     TIMER + 2, AL
        jmp     SHORT $+2

        mov     AL, CH          ;  Enable the Timer
        or      AL, 1
        out     GATE, AL

_DLAY_LOOP:                     ;  Wait for the Timer to Match

        in      AL, TIMER+2
        mov     AH, AL
        jmp     SHORT $+2
        in      AL, TIMER+2

        xchg    AH, AL

        CMP     AX, BX          ;  Are we at the End of the Delay?
        jl      _DLAY_LOOP

        mov     AL, CL          ;  Restore the Timer
        out     GATE, AL

        mov     AX, 0           ;  Everything Ran Okay
_DLAY_End:                      ;  Return to Calling "C" Routine
        pop     dx              ;  Restore the Changed Registers
        pop     cx
        pop     bx
        pop     ax
        mov     sp, bp          ;  Restore the Pointer to the Passed Parameters
        pop     bp
        ret                     ;  Return to Caller
_DLAY   ENDP


PAGE+
;  Variables


PASMPROCS ENDS
        END
