//  Graphic4 - Put in a Moving Pallette Register with Meesage
//
//  This Program Does Colour Palette Rotation Based on the
//   PC's Real Time Clock.  A Message is on the Screen to show
//   off what is possible.
//
//  Myke Predko
//  98.11.16
//

//  Defines
#include <stdio.h>
#include <dos.h>
#include <asmprocs.h>

#define VidInt 0x010            //  Video Interrupt

struct pixel                    //  Value to Put on the Screen
{
  char character;               //  The Character
  char colour;                  //  Colour Attributes
};

struct palette                  //  Value to Save Pallet Registers
{
  char Red;
  char Green;
  char Blue;
};

//  Global Variables
struct pixel Original[ 2000 ];  //  Array Used for the Original Screen
int  OrigPos, OrigCurs;         //  Save the Cursor Position

struct pixel far * VRAM;        //  Pointer to the Video RAM Buffer

struct palette PalData[ 256 ];  //  Data to Save the Palette In

char far * Pixels;              //  Pointer to Pixels on the Display

char FuncTable[ 64 ];


//  Subroutines
DisplayString( char far * String, int Colour )
{                               //  Display the String on the Display

union REGS regs;

int  i = 0;


  while ( String[ i ] != '\0' )
    {
      regs.h.ah = 0x009;        //  Send the String to the Display
      regs.h.al = String[ i++ ];
      regs.h.bh = Colour;
      regs.h.bl = ( Colour + 160 ) & 0x0FF;
      regs.x.cx = 1;
      int86( VidInt, &regs, &regs );

      regs.h.ah = 3;            //  Increment the Cursor Position
      regs.x.bx = 0;
      int86( VidInt, &regs, &regs );
      regs.h.dl++;
      regs.h.ah = 2;
      int86( VidInt, &regs, &regs );
    }  /*  endwhile  */
}  //  End DisplayString

//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

union REGS regs;
struct SREGS sregs;

int  i, j;
int  far * ClkData;             //  Pointer to the Clock Data

struct palette SaveData;


  regs.h.ah = 3;                //  Save the Current Cursor Position
  regs.h.bh = 0;
  int86( VidInt, &regs, &regs );
  OrigPos = regs.x.dx;
  OrigCurs = regs.x.cx;

  (long) VRAM = 0x0B8000000L;   //  Save the Current Screen
  for ( i = 0; i < 2000; i++ )
    Original[ i ] = VRAM[ i ];

  regs.h.ah = 0;                //  Display the Graphics Screen
  regs.h.al = 0x013;
  int86( VidInt, &regs, &regs );

  for ( i = 0; i < 256; i++ ) { //  Read/Save the Palette Registers
    regs.h.ah = 0x010;          //  Read the Register Values
    regs.h.al = 0x015;
    regs.x.bx = i;
    int86( VidInt, &regs, &regs );
    PalData[ i ].Red = regs.h.dh;
    PalData[ i ].Green = regs.h.ch;
    PalData[ i ].Blue = regs.h.cl;
    if ( i < 64 )               //  Grey for a Start
      regs.h.dh = regs.h.ch = regs.h.cl = i;
    else
      if ( i < 128 ) {          //  Red for Next;
        regs.h.dh = ( i & 0x03F );
        regs.h.ch = regs.h.cl = 0;
      } else
        if ( i < 192 ) {        //  Green for Next
          regs.h.ch = ( i & 0x03F );
          regs.h.dh = regs.h.cl = 0;
        } else {                //  Blue for the Rest
          regs.h.cl = ( i & 0x03F );
          regs.h.dh = regs.h.ch = 0;
        }  //  endif
    regs.h.ah = 0x010;          //  Put in the New Palette Value
    regs.h.al = 0x010;
    regs.x.bx = i;
    int86( VidInt, &regs, &regs );
  }  /*  endfor  */

  (long) Pixels = 0x0A0000000L; //  Display the Default Pixel Colours
  for ( i = 0; i < 200; i++ )
    for ( j = 0; j < 320; j++ )
      Pixels[( i * 320 ) + j ] = ( i + j ) & 0x0FF;

  regs.h.ah = 0x002;            //  Put a Message On
  regs.x.bx = 0;
  regs.h.dh = 2;
  regs.h.dl = 8;
  int86( VidInt, &regs, &regs );

  DisplayString( "Colour Palette Rotation", 0 );

  while ( GETKEYSTAT() != 0 )   //  Wait for a KeyPress
    GETKEY();

  (long) ClkData = 0x00040006CL;
  j = *ClkData;
  while ( GETKEYSTAT() == 0 )   //  Wait for a KeyPress
    if ( j != *ClkData ) {      //  Update the Palette
      regs.h.ah = 0x010;        //  Read Palette Register Zero
      regs.h.al = 0x015;
      regs.x.bx = 0;
      int86( VidInt, &regs, &regs );
      SaveData.Red = regs.h.dh;
      SaveData.Green = regs.h.ch;
      SaveData.Blue = regs.h.cl;
      for ( i = 0; i < 255; i++ ) {
        regs.h.ah = 0x010;      //  Read the Next Register
        regs.h.al = 0x015;
        regs.x.bx = i + 1;
        int86( VidInt, &regs, &regs );
        regs.h.ah = 0x010;      //  Store it in the Current
        regs.h.al = 0x010;
        regs.x.bx = i;
        int86( VidInt, &regs, &regs );
      }  //  endfor
      regs.h.ah = 0x010;        //  Put the First Register
      regs.h.al = 0x010;        //  Into the Last
      regs.x.bx = 0x0FF;
      regs.h.dh = SaveData.Red;
      regs.h.ch = SaveData.Green;
      regs.h.cl = SaveData.Blue;
      int86( VidInt, &regs, &regs );

      j = *ClkData;
    }  //  endif

  GETKEY();                     //  Clear it Out of the Buffer

  for ( i = 0; i < 256; i++ ) { //  Restore the Pixel Colours
    regs.h.ah = 0x010;          //  Read the Register Values
    regs.h.al = 0x010;
    regs.x.bx = i;
    regs.h.dh = PalData[ i ].Red;
    regs.h.ch = PalData[ i ].Green;
    regs.h.cl = PalData[ i ].Blue;
    int86( VidInt, &regs, &regs );
  }  /*  endfor  */

  regs.h.ah = 0;                //  Restore the Display
  regs.h.al = 3;
  int86( VidInt, &regs, &regs );

  regs.h.ah = 1;                //  Restore the Cursor Type
  regs.x.cx = OrigCurs;
  int86( VidInt, &regs, &regs );

  regs.h.ah = 2;                //  Restore the Cursor Position
  regs.h.bh = 0;
  regs.x.dx = OrigPos;
  int86( VidInt, &regs, &regs );

  for ( i = 0; i < 2000; i++ )  //  Restore the Display Values
    VRAM[ i ] = Original[ i ];

}  //  End Graphic3
